/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <pwd.h>
#include "main.h"
#include "misc.h"
#include "diagram.h"
#include "router.h"
#include "machine.h"
#include "network.h"
#include "interface.h"
#include "subnet.h"

Files::Files(Main *m)
{
	struct passwd *p;
	int i;

	p = getpwuid(getuid());
	sprintf(current_view, "%s/%s/views/default", p->pw_dir, HOME_DIR);

	update_in_progress = 0;
	main = m;
	objects = NULL;
	newobj = NULL;
	new_view = 0;

	for (i=0; i<MAX_TIMESTAMP; i++)
		timestamp[i] = 0;
}

Files::~Files()
{
	delete objects;
}

int Files::check()
{
	struct stat statbuf;
	char buf[256];
	int rt, s, c=0;
	struct passwd *p;
	Router *r;
	Link *ln;

	if (update_in_progress == 1) { printf("OVERLAP\n"); return 0; }
	update_in_progress = 1;
	p = getpwuid(getuid());
	sprintf(buf, "%s/%s", p->pw_dir, HOME_DIR);
	if (stat(buf, &statbuf) == -1) mkdir(buf, 493); // rwxr xr x
	sprintf(buf, "%s/%s/views", p->pw_dir, HOME_DIR);
	if (stat(buf, &statbuf) == -1) mkdir(buf, 493); // rwxr xr x
//	sprintf(buf, "%s/%s/help", p->pw_dir, HOME_DIR);
//	if (stat(buf, &statbuf) == -1) mkdir(buf, 493); // rwxr xr x

	sprintf(buf, "%s/network", main->pref->server_directory);
	stat(buf, &statbuf);
	if (statbuf.st_mtime != timestamp[0])
	{
		topology(); timestamp[0] = statbuf.st_mtime; c=1; clear_tags();
		if ((main->pref->auto_bunch) &&
				(objects!=NULL) && (main->monitor->animate==0))
		{
			for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
			{
				r = (Router*) ln->item;
				buncha(r);
			}
			main->monitor->select_all(0);
		}
	}

	// load current view
	sprintf(buf, "%s", current_view);

	sprintf(buf, "%s/ping", main->pref->server_directory);
	stat(buf, &statbuf);
	if (statbuf.st_mtime != timestamp[1])
	{ status(); timestamp[1] = statbuf.st_mtime; c=1; }

	update_in_progress = 0;

	if (objects==NULL) return c;
	rt = s = 0;
	for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if (r->type == 'r') rt++; else
		if (r->type == 's') s++;
	}
	sprintf(buf, "Routers %d\nSwitches %d\nMachines %d\nSubnets %d", rt, s, objects->machines.size(), objects->subnets.size());
	gtk_label_set_text(GTK_LABEL(main->monitor->label34), buf);

	return c;
}

void Files::status()
{
	FILE *f;
	char buf[256];
	float ts;
	Router *r;
	IPAddress tmpip;
	Link *ln;

	if (objects==NULL) return;

	for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		r->status = 0;
	}

	sprintf(buf, "%s/ping", main->pref->server_directory);
	f = fopen(buf, "r");
	if (!f) return;

	for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		fseek(f, 0L, SEEK_SET);
		fscanf(f, "%[^\n]\n", buf); // the date stamp

		while(fscanf(f, "%[^ ] (%f ms)\n", buf, &ts)==2)
		{
			tmpip.set(buf);
			if (tmpip.compare(r->pingname))
				r->status = 1;
		}
	}
	fclose(f);
}

void Files::topology()
{
//	FILE *f;
	char buf[256];
	Link *ln, *ln2, *ln3, *ln4;
	Router *r, *r2;
	Interface *in, *in2;

	sprintf(buf, "%s/network", main->pref->server_directory);

/*
	f = fopen(buf, "r");
	if (!f) return;
	newobj = new Data();
	newobj->load(f);
*/

	newobj = new Data();
	newobj->loadxml(buf);

	// copy old coordinates across
	if (objects==NULL)
	{
		objects = newobj;
		load_view(current_view);
	}else
	{
		for (ln=newobj->routers.get_head(); ln!=NULL; ln=ln->next)
		{
			r = (Router*) ln->item;
			for (ln2=objects->routers.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				r2 = (Router*) ln2->item;
				if (strcmp(r2->name, r->name)==0)
				{ 
					r->status = r2->status;
					r->x = r2->x;
					r->y = r2->y;
					r->w = r2->w;
					r->h = r2->h;
					r->selected = r2->selected;
					r->show = r2->show;
					r->visited = r2->visited;
					r->find = r2->find;
					r->fix = r2->fix;

					for (ln3=r->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
					{
						in = (Interface*) ln3->item;
						for (ln4=r2->interfaces.get_head(); ln4!=NULL; ln4=ln4->next)
						{
							in2 = (Interface*) ln4->item;
							if ((in->index == in2->index) &&
									(in->network!=NULL) && (in2->network!=NULL))
							{
								in->network->view = in2->network->view;
								in->network->x = in2->network->x;
								in->network->y = in2->network->y;
								in->network->w = in2->network->w;
								in->network->h = in2->network->h;
								in->network->selected = in2->network->selected;
								in->network->show = in2->network->show;
								in->network->visited = in2->network->visited;
								in->network->find = in2->network->find;
								in->network->fix = in2->network->fix;
							}
						}
					}
				}
			}
		}

// note lots of stuff

		delete objects;
		objects = newobj;

// update stuff here

	}
//	fclose(f);
}

void Files::load_view(char *fname)
{
	char buf[256], val[256];
	int v1, v2, v3;
	Link *ln, *ln2;
	Router *r;
	Interface *in;
	char nm;

	f = fopen(fname, "r");
	if (!f) return;
	if (objects==NULL) return;

	strcpy(current_view, fname);
	sprintf(buf, "Network Monitor - %s", fname);
  gtk_window_set_title(GTK_WINDOW(main->monitor->window1), buf);

	while(fscanf(f, "%c %[^\n]\n", &nm, val)==2)
	{
		if (nm == 'L')
		{
			if (strcmp(val, "on")==0) main->monitor->diagram->labels = 1;
			else  main->monitor->diagram->labels = 0;
		}else
		if (nm == 'G')
		{
			if (strcmp(val, "on")==0)  main->monitor->grid = GRID_SIZE;
			else  main->monitor->grid = 0;
		}else
		if (nm == 'S')
		{
			if (strcmp(val, "on")==0)  main->monitor->snap = 1;
			else  main->monitor->snap = 0;
		}else
		if (nm == 'R')
		{
			if (sscanf(val, "%[^ ] %d %d", buf, &v1, &v2)==3)
			{
				for (ln = objects->routers.get_head(); ln!=NULL; ln=ln->next)
				{
					r = (Router*) ln->item;
					if (strcmp(r->name, buf)==0)
					{
						r->x = v1; r->y = v2;
					}
				}
			}
		}
		if (nm == 'N')
		{
			if (sscanf(val, "%[^ ] %d %d %d", buf, &v1, &v2, &v3)==4)
			{
				for (ln = objects->routers.get_head(); ln!=NULL; ln=ln->next)
				{
					r = (Router*) ln->item;
					if (strcmp(r->name, buf)==0)
					{
						for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
						{
							in = (Interface*) ln2->item;
							if (in->index == v1)
							{
								if (in->network!=NULL)
								{
									in->network->x = v2;
									in->network->y = v3;
								}
							}
						}
					}
				}
			}
		}
	}
	fclose(f);
	main->monitor->draw();
}

void Files::save_view(char *fname)
{
	Link *ln, *ln2;
	Router *r;
	Network *n;
	Interface *in;

	f = fopen(fname, "w");
	if (!f) return;

	if (main->monitor->diagram->labels == 1) fprintf(f, "L on\n");
	else fprintf(f, "L off\n");
	if (main->monitor->grid == GRID_SIZE) fprintf(f, "G on\n");
	else fprintf(f, "G off\n");
	if (main->monitor->snap == 1) fprintf(f, "S on\n");
	else fprintf(f, "S off\n");

	for (ln = objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		fprintf(f, "R %s %d %d\n", r->name, (int) (r->x), (int) (r->y));
	}

	for (ln = objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		ln2 = n->interfaces.get_head();
		if (ln2!=NULL)
		{
			in = (Interface*) ln2->item;
			fprintf(f, "N %s %d %d %d\n",
				in->router->name, in->index, (int) (n->x), (int) (n->y));
		}
	}

// also store filter values & apply when reloaded to set show flags

	fclose(f);
}

void Files::print_view(char *fname, int pw, int ph, int nw, int nh, int bw, int bh)
{
/*
	FILE *f;
	time_t now;
	int i, j, counter;
	struct passwd *p;
	double rw, rh, ratio;
	Link *ln, *ln2;
	Router *r;
	Machine *m;
	Network *n;
	Interface *in;
	int counts;
	Subnet *sn;
	char nm[256], tmpnm[256];

	now = time(NULL);
	p = getpwuid(getuid());
	main->monitor->height = 0;
	main->monitor->width = 0;
	main->monitor->compress();
	rw = pw*nw/(float)main->monitor->width;
	rh = ph*nh/(float)main->monitor->height;
	if (rw<rh) ratio = rw; else ratio = rh;

	f = fopen(fname, "w");
	if (!f) return;

	fprintf(f, "%%!PS-Adobe-2.0\n");
	fprintf(f, "%%%%DocumentFonts: Helvetica\n");
	fprintf(f, "%%%%Title: Network Monitor - %s\n", current_view);
	fprintf(f, "%%%%Creator: %s (%s)\n", p->pw_gecos, p->pw_name);
	fprintf(f, "%%%%CreationDate: %s", asctime(localtime(&now)));
	fprintf(f, "%%%%Pages: (atend)\n");
	fprintf(f, "%%%%BoundingBox: 0 0 %d %d\n", ph, pw);
	fprintf(f, "%%%%EndComments\n\n");
	fprintf(f, "/mm {2.835 mul} def\n");
	fprintf(f, "/pagewidth {%d} def\n", pw);
	fprintf(f, "/pageheight {%d} def\n", ph);
	fprintf(f, "/leftmargin {%d} def\n", bw);
	fprintf(f, "/bottommargin {%d} def\n\n", bh);
	fprintf(f, "/red { 1 0 0 setrgbcolor } def\n");
	fprintf(f, "/green { 0 1 0 setrgbcolor } def\n");
	fprintf(f, "/blue { 0 0 1 setrgbcolor } def\n");
	fprintf(f, "/black { 0 0 0 setrgbcolor } def\n");
	fprintf(f, "/white { 1 1 1 setrgbcolor } def\n\n");

	fprintf(f, "/title\n");
	fprintf(f, "{   gsave black\n");
	fprintf(f, "    /Helvetica-Bold findfont 24 scalefont setfont\n");
	fprintf(f, "    20 -%d moveto (Network Monitor) show\n", (nh*ph)-80);
	fprintf(f, "    /Helvetica findfont 14 scalefont setfont\n");
	fprintf(f, "    20  -%d moveto (View:) show\n", (nh*ph)-40);
	fprintf(f, "    100 -%d moveto (%s) show\n", (nh*ph)-40, current_view);
	fprintf(f, "    20  -%d moveto (Date:) show\n", (nh*ph)-25);
	fprintf(f, "    100 -%d moveto (%s) show\n", (nh*ph)-25, asctime(localtime(&now)));
	fprintf(f, "    20  -%d moveto (By:) show\n", (nh*ph)-10);
	fprintf(f, "    100 -%d moveto (%s (%s)) show\n", (nh*ph)-10, p->pw_gecos, p->pw_name);
	fprintf(f, "    grestore\n");
	fprintf(f, "} def\n\n");

	fprintf(f, "%%%%EndProlog\n\n");

	fprintf(f, "/map\n{\n");
	fprintf(f, "  gsave\n");

	// connecting lines
	for (ln=objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->show==1)
		{
			for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				m = in->machine;
				if ((m->type == 'r') || (m->type == 's'))
				{
					fprintf(f, "black\n");
					fprintf(f, "%d -%d moveto %d -%d lineto stroke\n",
						(int) (m->x+(m->w/2)),
						(int) (m->y+(m->h/2)), (int) (n->x+(n->w/2)),
						(int) (n->y+(n->h/2)));
				}
			}
		}
	}

	// clouds
	for (ln=objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->show==1)
		{
			if (n->view == 0)
			{
				fprintf(f, "blue\n");
				fprintf(f, "%d -%d %d 0 360 arc fill\n", (int) (n->x+(n->w/2)),
					(int) (n->y+(n->h/2)), (int) (n->w/2));
			}else
			if (n->view == 1)
			{
				counts = n->subnets.size();
				n->w = 100;
				n->h = (counts*10)+4;		// 10 point font
				fprintf(f, "white\n");
				fprintf(f, "newpath %d -%d moveto %d 0 rlineto 0 -%d rlineto -%d 0 rlineto closepath fill\n", (int) (n->x), (int) (n->y), (int) (n->w), (int) (n->h), (int) (n->w));
				fprintf(f, "black\n");
				fprintf(f, "newpath %d -%d moveto %d 0 rlineto 0 -%d rlineto -%d 0 rlineto closepath stroke\n", (int) (n->x), (int) (n->y), (int) (n->w), (int) (n->h), (int) (n->w));
				i = 1;
				for (ln2=n->subnets.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					sn = (Subnet*) ln2->item;
					if (strlen(sn->name)>0) strcpy(nm, sn->name);
					else
					{
						sprint_ip(sn->ip, tmpnm);
						sprintf(nm, "%s / %d", tmpnm, sn->mask);
					}
					fprintf(f, "/Helvetica-Bold findfont 10 scalefont setfont\n");
					fprintf(f, "gsave (%s) %d -%d moveto show grestore\n", nm,
						(int) (n->x + 2),
						(int) (n->y + 2 + (i*10)));
					i++;
				}
			}else
			{
				counts = n->num_machines();
				n->w = 100;
				n->h = (counts*10)+4;		// 10 point font
				fprintf(f, "white\n");
				fprintf(f, "newpath %d -%d moveto %d 0 rlineto 0 -%d rlineto -%d 0 rlineto closepath fill\n", (int) (n->x), (int) (n->y), (int) (n->w), (int) (n->h), (int) (n->w));
				fprintf(f, "black\n");
				fprintf(f, "newpath %d -%d moveto %d 0 rlineto 0 -%d rlineto -%d 0 rlineto closepath stroke\n", (int) (n->x), (int) (n->y), (int) (n->w), (int) (n->h), (int) (n->w));
				i = 1;
				for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					m = in->machine;
					if (m->type == 'm')
					{
						if (strlen(m->name)>0) strcpy(nm, m->name); else
						if (zero_ip(m->ip)==0) sprint_ip(m->ip, nm); else
							sprint_mac(in->mac, nm);

						fprintf(f, "/Helvetica-Bold findfont 10 scalefont setfont\n");
						fprintf(f, "gsave (%s) %d -%d moveto show grestore\n", nm,
							(int) (n->x + 2),
							(int) (n->y + 2 + (i*10)));
						i++;
					}
				}
			}
		}
	}

	// devices
	for (ln = objects->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if ((m->show == 1) && ((m->type=='r') || (m->type=='s')))
		{
			fprintf(f, "green\n");
			fprintf(f, "%d -%d %d 0 360 arc fill\n", (int) (m->x+(m->w/2)), (int) (m->y+(m->h/2)), (int) (m->w/2));
		}
	}

	// device names
	if (main->monitor->labels == 1)
	{
		for (ln = objects->machines.get_head(); ln!=NULL; ln=ln->next)
		{
			m = (Machine*) ln->item;
			if ((m->show == 1) && ((m->type=='r') || (m->type=='s')))
			{
				fprintf(f, "black\n");
				fprintf(f, "/Helvetica-Bold findfont 10 scalefont setfont\n");
				fprintf(f, "gsave (%s) dup stringwidth pop 2 div %d exch sub -%d moveto show grestore\n",
					m->name, (int) (m->x+8), (int) (m->y+16+4+10));	// 10 point font
			}
		}
	}

	fprintf(f, "  grestore\n");
	fprintf(f, "} def\n\n");

	fprintf(f, "0 pageheight %d mul translate\n\n", nh);

	for (i=0; i<nw; i++)
	{
		for (j=0; j<nh; j++)
		{
			fprintf(f, "%%%%Page: %d %d\n", (nh*i)+j+1, (nh*i)+j+1);
			fprintf(f, "gsave\n");
			// adjust for margin
			fprintf(f, "  %d %d translate\n", bw, bh);
			if (main->print->orientation == 1)
				fprintf(f, "  90 rotate %d neg %d translate\n", (ph*nh)+(pw*i), (ph*(nh-1-j)));
			else
				fprintf(f, "  %d %d translate\n", -(i*pw), -(j*ph));
			if ((i == 0) && (j == 0)) fprintf(f, "  title\n");
			fprintf(f, "newpath\n");
			fprintf(f, "  %d %d moveto\n", i*pw, -(nh-j)*ph);
			fprintf(f, "  0 pageheight rlineto\n");
			fprintf(f, "  pagewidth 0 rlineto\n");
			fprintf(f, "  0 pageheight neg rlineto\n");
			fprintf(f, "closepath clip .1 setgray stroke\n\n");
			fprintf(f, "  %f %f scale\n", ratio, ratio);
			fprintf(f, "  leftmargin bottommargin moveto map gsave showpage grestore\n");
			fprintf(f, "grestore\n\n");
			counter++;
		}
	}
	fprintf(f, "%%%%Pages: %d\n", nw*nh);

	fclose(f);
*/
}

void Files::stara(Router *r)
{
	Link *ln2;
	double radius, angle;
	int l;
	Interface *in;

	r->selected = 1;

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if (in->network!=NULL)
		{
			if (in->network->interfaces.size() == 1) l++;
		}
	}

	if (l<8) l=8;
	radius = (l * 10)/(2*3.14159);
	angle = 2*3.14159/l;

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if (in->network!=NULL)
		{
			if (in->network->interfaces.size() == 1)
			{
				in->network->x = r->x + (r->w/2) - in->network->w/2 +
					((int) (radius * cos(angle*l)));
				in->network->y = r->y + (r->h/2) - in->network->h/2 +
					((int) (radius * sin(angle*l)));
				in->network->selected = 1;
				l++;
			}
		}
	}
}

void Files::star()
{
	Link *ln;
	Router *r;

	// take clouds as 16 across and devices as 32 across (twice actual sizes)
	// say we have a device at the center of the star and 19 clouds connected
	// we take 19*16 = 304 which is the circumferance of the circle. The radius
	// will therefore be 304/(3.142*2) = 48.377 The angle we need to turn
	// through will be 360/19 = 18.947 degrees

	if (main->monitor->current != NULL)
	{
		if ((main->monitor->current->type == 'r') ||
				(main->monitor->current->type == 's'))
		{
			if (main->monitor->current->selected == 0)
			{
				// deselect everything else first
				main->monitor->select_all(0);
				r = (Router*) main->monitor->current;
				stara(r);
			}
		}
	}
	for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if (r->selected == 1)
			stara(r);
	}
}

void Files::buncha(Router *r)
{
	Link *ln2;
	double radius, angle;
	int l, total, ring;
	Interface *in;

	r->selected = 1;

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if (in->network != NULL)
		{
			total = ring = 0;
			while (total<l+1) { ring++; total += (8*ring); }

			radius = (((8*ring)+4) * 8)/(2*3.14159);
			angle = 2*3.14159/(ring*8);

			if (in->network->interfaces.size() == 1)
			{
				in->network->x = r->x + (r->w/2) - in->network->w/2 +
					((int) (radius * cos(angle*(total-l))));
				in->network->y = r->y + (r->h/2) - in->network->h/2 +
					((int) (radius * sin(angle*(total-l))));
				in->network->selected = 1;
				l++;
			}
		}
	}
}

void Files::bunch()
{
	Link *ln;
	Router *r;

	// take clouds as 16 across and devices as 32 across (twice actual sizes)
	// say we have a device at the center of the star and 19 clouds connected
	// we take 19*16 = 304 which is the circumferance of the circle. The radius
	// will therefore be 304/(3.142*2) = 48.377 The angle we need to turn
	// through will be 360/19 = 18.947 degrees

	if (main->monitor->current != NULL)
	{
		if ((main->monitor->current->type == 'r') ||
				(main->monitor->current->type == 's'))
		{
			if (main->monitor->current->selected == 0)
			{
				// deselect everything else first
				main->monitor->select_all(0);
				r = (Router*) main->monitor->current;
				buncha(r);
			}
		}
	}
	for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if (r->selected == 1)
			buncha(r);
	}
}

void Files::width_height()
{
	Link *ln;
	Network *n;

	if (objects==NULL) return;

	for (ln=objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->view == 0)
		{
			n->w = 8;
			n->h = 8;
		}else
		if (n->view == 1)
		{
			if (n->subnets.size()==0)
			{
				n->w = 8;
				n->h = 8;
			}else
			{
				n->w = 100;
				n->h = (n->subnets.size()*main->font10b->ascent)+4;
			}
		}else
		{
			n->w = 100;
			n->h = (n->machines.size()*main->font10b->ascent)+4;
		}
	}
}

void Files::shift(int xx, int yy)
{
	Link *ln;
	Router *r;
	Network *n;

	if (objects==NULL) return;

	for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		r->x += xx;
		r->y += yy;
	}
	for (ln=objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->x += xx;
		n->y += yy;
	}
}

void Files::clear_tags()
{
	Link *ln;
	Router *r;
	Network *n;

	if (objects==NULL) return;

	for (ln=objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
//		r->tag=0;
		r->selected=0;
	}
	for (ln=objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
//		n->tag=0;
		n->selected=0;
	}
}

