/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "main.h"
#include "filterwindow.h"
#include "filter.h"
#include "images/icon.xpm"
#include "images/up.xpm"
#include "images/down.xpm"

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((FilterWindow*)data)->hide();
	return (TRUE); 
}

static void new_filter(GtkWidget *widget, gpointer data)
{
	FilterWindow *fw = (FilterWindow*) data;
	Filter *f;
	gchar *rowdata[1];
	gint row;
	char buf[256];

	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(fw->entry12)));

	if (strcmp(buf, "")==0) return;

	f = new Filter();
	strcpy(f->name, buf);
	fw->main->files->filters.add(f);

	if (fw->filter_row != -1)
	{
		gtk_clist_unselect_row(GTK_CLIST(fw->clist21), fw->filter_row, 0);
		fw->filter_row = -1;
	}

	rowdata[0] = strdup(f->name);
	row = gtk_clist_append(GTK_CLIST(fw->clist21), rowdata);
	delete rowdata[0];
	gtk_clist_set_row_data(GTK_CLIST(fw->clist21), row, f);
	gtk_entry_set_text(GTK_ENTRY(fw->entry12), "");
}

static void select_filter(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	FilterWindow *fw = (FilterWindow*) data;
	Filter *f;

	f = (Filter*) gtk_clist_get_row_data(GTK_CLIST(fw->clist21), row);
	fw->filter_row = row;
	gtk_entry_set_text(GTK_ENTRY(fw->entry12), f->name);
	gtk_widget_set_sensitive(fw->frame10, TRUE);
}

static void deselect_filter(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	FilterWindow *fw = (FilterWindow*) data;

	fw->filter_row = -1;
	gtk_entry_set_text(GTK_ENTRY(fw->entry12), "");
	gtk_widget_set_sensitive(fw->frame10, FALSE);
}

static void select_entry(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
printf("SELECT\n");
}

static void deselect_entry(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
printf("DESELECT\n");
}

static void delete_filter(GtkWidget *widget, gpointer data)
{
	FilterWindow *fw = (FilterWindow*) data;
	Filter *f;

	if (fw->filter_row != -1)
	{
		f = (Filter*) gtk_clist_get_row_data(GTK_CLIST(fw->clist21), fw->filter_row);
		fw->main->files->filters.del(f);
		delete f;
		gtk_clist_remove(GTK_CLIST(fw->clist21), fw->filter_row);
		fw->filter_row = -1;
	}
}

static void edit_filter(GtkWidget *widget, gpointer data)
{
	FilterWindow *fw = (FilterWindow*) data;
	Filter *f;

	if (fw->filter_row != -1)
	{
		f = (Filter*) gtk_clist_get_row_data(GTK_CLIST(fw->clist21), fw->filter_row);
		
	}
}

static void key_delete_filter(GtkWidget *w, GdkEventKey *event, gpointer data)
{
	if (event->keyval == 0xffff)
		delete_filter(w, data);
}

static void key_delete_entry(GtkWidget *w, GdkEventKey *event, gpointer data)
{
	printf("KEY DELETE\n");
	//	if (event->keyval == 0xffff)
	//		((Monitor*)data)->del();
}

static void hide_entry(GtkWidget *widget, gpointer data)
{
printf("HIDE\n");
}

/*
static void show_entry(GtkWidget *widget, gpointer data)
{
printf("SHOW\n");
}
*/

static void delete_entry(GtkWidget *widget, gpointer data)
{
printf("DEL\n");
}

static void up_entry(GtkWidget *widget, gpointer data)
{
printf("UP\n");
}

static void down_entry(GtkWidget *widget, gpointer data)
{
printf("DOWN\n");
}

static void edit_entry(GtkWidget *widget, gpointer data)
{
printf("EDIT\n");
}

static void copy_filter(GtkWidget *widget, gpointer data)
{
printf("COPY\n");
}

static void include_path(GtkWidget *w, gpointer data)
{
}

static void save_cb(GtkWidget *w, gpointer data)
{
	((FilterWindow*)data)->hide();
}
static void close_cb(GtkWidget *w, gpointer data)
{
	((FilterWindow*)data)->hide();
}

FilterWindow::FilterWindow(Main *m)
{
	main = m;
	filter_row = -1;
	entry_row = -1;
	create();
}

FilterWindow::~FilterWindow()
{
}

void FilterWindow::create()
{
  filter = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (filter), "Network Filter");
  gtk_widget_realize (filter);
	window_icon(filter, icon_xpm);
	gdk_window_set_icon_name(filter->window, "Filter");
  gtk_widget_set_usize (filter, 400, 240);
	gtk_signal_connect(GTK_OBJECT(filter), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  pixmap[0] = create_pixmap(filter, up_xpm);
  pixmap[1] = create_pixmap(filter, down_xpm);

  vbox33 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox33);
  gtk_container_add (GTK_CONTAINER (filter), vbox33);
  gtk_container_set_border_width (GTK_CONTAINER (vbox33), 4);

  hbox47 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox47);
  gtk_box_pack_start (GTK_BOX (vbox33), hbox47, TRUE, TRUE, 0);

  frame9 = gtk_frame_new ("Filter List");
  gtk_widget_show (frame9);
  gtk_box_pack_start (GTK_BOX (hbox47), frame9, FALSE, TRUE, 0);

  vbox32 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox32);
  gtk_container_add (GTK_CONTAINER (frame9), vbox32);
  gtk_container_set_border_width (GTK_CONTAINER (vbox32), 4);

  scrolledwindow22 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow22);
  gtk_box_pack_start (GTK_BOX (vbox32), scrolledwindow22, TRUE, TRUE, 0);
  gtk_widget_set_usize (scrolledwindow22, 120, -2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow22),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist21 = gtk_clist_new (1);
  gtk_widget_show (clist21);
  gtk_container_add (GTK_CONTAINER (scrolledwindow22), clist21);
  gtk_widget_set_usize (clist21, 120, -2);
  gtk_clist_set_column_width (GTK_CLIST (clist21), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (clist21));
	gtk_signal_connect(GTK_OBJECT(clist21), "select_row",
		GTK_SIGNAL_FUNC(select_filter), this);
	gtk_signal_connect(GTK_OBJECT(clist21), "unselect_row",
		GTK_SIGNAL_FUNC(deselect_filter), this);
	gtk_signal_connect(GTK_OBJECT(clist21), "key-press-event",
		GTK_SIGNAL_FUNC(key_delete_filter), this);

  entry12 = gtk_entry_new ();
  gtk_widget_show (entry12);
  gtk_box_pack_start (GTK_BOX (vbox32), entry12, FALSE, FALSE, 0);
  gtk_widget_set_usize (entry12, 120, -2);
	gtk_signal_connect(GTK_OBJECT(entry12), "activate",
		GTK_SIGNAL_FUNC(new_filter), this);

  hbox48 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox48);
  gtk_box_pack_start (GTK_BOX (vbox32), hbox48, FALSE, TRUE, 0);

  button44 = gtk_button_new_with_label ("Add");
  gtk_widget_show (button44);
  gtk_box_pack_start (GTK_BOX (hbox48), button44, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button44), "clicked",
		GTK_SIGNAL_FUNC (new_filter), this);

  button45 = gtk_button_new_with_label ("Delete");
  gtk_widget_show (button45);
  gtk_box_pack_start (GTK_BOX (hbox48), button45, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button45), "clicked",
		GTK_SIGNAL_FUNC (delete_filter), this);

  hbox64 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox64);
  gtk_box_pack_start (GTK_BOX (vbox32), hbox64, FALSE, TRUE, 0);

  button61 = gtk_button_new_with_label ("Edit");
  gtk_widget_show (button61);
  gtk_box_pack_start (GTK_BOX (hbox64), button61, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button61), "clicked",
		GTK_SIGNAL_FUNC (edit_filter), this);

  button62 = gtk_button_new_with_label ("Copy");
  gtk_widget_show (button62);
  gtk_box_pack_start (GTK_BOX (hbox64), button62, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button62), "clicked",
		GTK_SIGNAL_FUNC (copy_filter), this);

  frame10 = gtk_frame_new ("Filter");
  gtk_widget_show (frame10);
  gtk_box_pack_start (GTK_BOX (hbox47), frame10, TRUE, TRUE, 0);

  vbox34 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox34);
  gtk_container_add (GTK_CONTAINER (frame10), vbox34);
  gtk_container_set_border_width (GTK_CONTAINER (vbox34), 4);

  scrolledwindow23 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow23);
  gtk_box_pack_start (GTK_BOX (vbox34), scrolledwindow23, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow23),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist22 = gtk_clist_new (2);
  gtk_widget_show (clist22);
  gtk_container_add (GTK_CONTAINER (scrolledwindow23), clist22);
  gtk_widget_set_usize (clist22, -2, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist22), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist22), 1, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (clist22));
	gtk_signal_connect(GTK_OBJECT(clist22), "select_row",
		GTK_SIGNAL_FUNC(select_entry), this);
	gtk_signal_connect(GTK_OBJECT(clist22), "unselect_row",
		GTK_SIGNAL_FUNC(deselect_entry), this);
	gtk_signal_connect(GTK_OBJECT(clist22), "key-press-event",
		GTK_SIGNAL_FUNC(key_delete_entry), this);

  entry13 = gtk_entry_new ();
  gtk_widget_show (entry13);
  gtk_box_pack_start (GTK_BOX (vbox34), entry13, FALSE, FALSE, 0);

  hbox49 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox49);
  gtk_box_pack_start (GTK_BOX (vbox34), hbox49, FALSE, TRUE, 0);

  button46 = gtk_button_new_with_label ("Add");
  gtk_widget_show (button46);
  gtk_box_pack_start (GTK_BOX (hbox49), button46, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button46), "clicked",
		GTK_SIGNAL_FUNC (hide_entry), this);

  button63 = gtk_button_new_with_label ("Edit");
  gtk_widget_show (button63);
  gtk_box_pack_start (GTK_BOX (hbox49), button63, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button63), "clicked",
		GTK_SIGNAL_FUNC (edit_entry), this);

  button47 = gtk_button_new_with_label ("Delete");
  gtk_widget_show (button47);
  gtk_box_pack_start (GTK_BOX (hbox49), button47, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button47), "clicked",
		GTK_SIGNAL_FUNC (delete_entry), this);

  hbox51 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox51);
  gtk_box_pack_start (GTK_BOX (vbox34), hbox51, FALSE, TRUE, 0);

  button50 = gtk_button_new ();
  gtk_widget_show (button50);
  gtk_box_pack_start (GTK_BOX (hbox51), button50, FALSE, FALSE, 0);
	gtk_container_add (GTK_CONTAINER (button50), pixmap[0]);
  gtk_signal_connect (GTK_OBJECT (button50), "clicked",
		GTK_SIGNAL_FUNC (up_entry), this);

  button51 = gtk_button_new ();
  gtk_widget_show (button51);
  gtk_box_pack_start (GTK_BOX (hbox51), button51, FALSE, FALSE, 0);
	gtk_container_add (GTK_CONTAINER (button51), pixmap[1]);
  gtk_signal_connect (GTK_OBJECT (button51), "clicked",
		GTK_SIGNAL_FUNC (down_entry), this);

  checkbutton40 = gtk_check_button_new_with_label ("Include Path");
  gtk_widget_show (checkbutton40);
  gtk_box_pack_end (GTK_BOX (hbox51), checkbutton40, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (checkbutton40), "clicked",
		(GtkSignalFunc) include_path, this);

  hbox50 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox50);
  gtk_box_pack_start (GTK_BOX (vbox33), hbox50, FALSE, TRUE, 0);

	drawingarea10 = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea10);
  gtk_box_pack_start (GTK_BOX (hbox50), drawingarea10, TRUE, TRUE, 0);

  button48 = gtk_button_new_with_label ("Save");
  gtk_widget_show (button48);
  gtk_box_pack_start (GTK_BOX (hbox50), button48, FALSE, FALSE, 0);
  gtk_widget_set_usize (button48, 100, -2);
  gtk_signal_connect (GTK_OBJECT (button48), "clicked",
		GTK_SIGNAL_FUNC (save_cb), this);

  button52 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button52);
  gtk_box_pack_end (GTK_BOX (hbox50), button52, FALSE, FALSE, 0);
  gtk_widget_set_usize (button52, 100, -2);
  gtk_signal_connect (GTK_OBJECT (button52), "clicked",
		GTK_SIGNAL_FUNC (close_cb), this);
}

void FilterWindow::show()
{
	gtk_widget_show(filter);
	gdk_window_raise(filter->window);
	init();
}

void FilterWindow::hide()
{
	gtk_widget_hide(filter);
}

void FilterWindow::init()
{
	gtk_widget_set_sensitive(frame10, FALSE);
}
