/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "iface.h"
#include "misc.h"
#include "router.h"
#include "machine.h"
#include "interface.h"
#include "images/octin.xpm"
#include "images/bcastin.xpm"
#include "images/disin.xpm"
#include "images/errin.xpm"
#include "images/octout.xpm"
#include "images/bcastout.xpm"
#include "images/disout.xpm"
#include "images/errout.xpm"
#include "images/icon.xpm"

int compare_router_names(Link *a, Link *b)
{
	Router *r1, *r2;

	r1 = (Router*) a->item;
	r2 = (Router*) b->item;

	return (strcmp(r1->name, r2->name));
}

void checkbox_cb(GtkWidget *widget, gpointer data)
{
	((Graph*)data)->draw();
}

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((Graph*)data)->hide();
}

static void router_cb(GtkWidget *w, gpointer data)
{
	((Graph*)data)->router();
	((Graph*)data)->load();
}

static void interface_cb(GtkWidget *w, gpointer data)
{
	((Graph*)data)->load();
}

static void spinner_cb(GtkWidget *w, gpointer data)
{
	((Graph*)data)->spinner();
}

static void scroll_cb(GtkWidget *w, gpointer data)
{
	((Graph*)data)->scroll();
}

static gint expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	Graph *m = (Graph*)data;

	if (m->canvas!=NULL)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			m->canvas, event->area.x, event->area.y, event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	return FALSE;
}

static gint configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer data)
{
	// Hmm the window has been resized so there will be an expose event along
	// in a minute to copy the canvas to the screen, but we need to redraw
	// the canvas as the size could have grown to fit the window
	((Graph*)data)->draw();
	return TRUE;
}

Graph::Graph(Main *m)
{
	main = m;
	canvas = NULL;
	iface = NULL;
	strcpy(current_file, "");
	create();

	gc      = gdk_gc_new(window1->window);
	gridgc  = gdk_gc_new(window1->window);
  gdk_gc_set_line_attributes(gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT,
		GDK_JOIN_MITER);

	init();
}

Graph::~Graph()
{
	if (gc) gdk_gc_unref(gc);
	if (gridgc) gdk_gc_unref(gridgc);
	if (iface != NULL)
		delete [] iface;
}

void Graph::create()
{
  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window1), "Network Graph");
  gtk_window_set_policy (GTK_WINDOW (window1), FALSE, TRUE, FALSE);
  gtk_widget_set_usize (window1, 400, 340);
  gtk_widget_realize (window1);
	window_icon(window1, icon_xpm);
	gdk_window_set_icon_name(window1->window, "Graph");

	pixmap3 = create_pixmap(window1, octin_xpm);
	pixmap4 = create_pixmap(window1, disin_xpm);
	pixmap5 = create_pixmap(window1, errin_xpm);
	pixmap6 = create_pixmap(window1, octout_xpm);
	pixmap7 = create_pixmap(window1, disout_xpm);
	pixmap8 = create_pixmap(window1, errout_xpm);
	pixmap9 = create_pixmap(window1, bcastin_xpm);
	pixmap10 = create_pixmap(window1, bcastout_xpm);

	vbox1 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 4);

  drawingarea1 = gtk_drawing_area_new ();
	gtk_widget_set_events (drawingarea1, GDK_EXPOSURE_MASK);
	gtk_widget_show (drawingarea1);
  gtk_box_pack_start (GTK_BOX (vbox1), drawingarea1, TRUE, TRUE, 0);

  hbox32 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox32);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox32, FALSE, TRUE, 0);

  frame6 = gtk_frame_new ("Router / Interface");
  gtk_widget_show (frame6);
  gtk_box_pack_start (GTK_BOX (hbox32), frame6, TRUE, TRUE, 0);

  vbox31 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox31);
  gtk_container_add (GTK_CONTAINER (frame6), vbox31);
  gtk_container_set_border_width (GTK_CONTAINER (vbox31), 4);

  combo3 = gtk_combo_new ();
  gtk_widget_show (combo3);
  gtk_box_pack_start (GTK_BOX (vbox31), combo3, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo3)->list), "selection_changed",
		GTK_SIGNAL_FUNC (router_cb), this);

  combo_entry2 = GTK_COMBO (combo3)->entry;
  gtk_widget_show (combo_entry2);
	gtk_entry_set_editable(GTK_ENTRY(combo_entry2), FALSE);

  combo4 = gtk_combo_new ();
  gtk_widget_show (combo4);
  gtk_box_pack_start (GTK_BOX (vbox31), combo4, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo4)->list), "selection_changed",
		GTK_SIGNAL_FUNC (interface_cb), this);

  combo_entry3 = GTK_COMBO (combo4)->entry;
  gtk_widget_show (combo_entry3);
	gtk_entry_set_editable(GTK_ENTRY(combo_entry3), FALSE);

  frame12 = gtk_frame_new ("Subnet List");
  gtk_widget_show (frame12);
  gtk_box_pack_start (GTK_BOX (hbox32), frame12, TRUE, TRUE, 0);

  scrolledwindow25 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow25);
  gtk_container_add (GTK_CONTAINER (frame12), scrolledwindow25);
  gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow25), 4);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow25),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  clist24 = gtk_clist_new (1);
  gtk_widget_show (clist24);
  gtk_container_add (GTK_CONTAINER (scrolledwindow25), clist24);
  gtk_clist_set_column_width (GTK_CLIST (clist24), 0, 80);
  gtk_widget_set_usize (clist24, -2, 30);
  gtk_clist_column_titles_hide (GTK_CLIST (clist24));

  frame5 = gtk_frame_new ("Show");
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (hbox32), frame5, FALSE, TRUE, 0);

	table4 = gtk_table_new (2, 4, FALSE);
  gtk_widget_show (table4);
  gtk_container_add (GTK_CONTAINER (frame5), table4);
  gtk_container_set_border_width (GTK_CONTAINER (table4), 4);

  checkbutton13 = gtk_check_button_new();
  gtk_widget_show (checkbutton13);
	gtk_container_add (GTK_CONTAINER (checkbutton13), pixmap3);
  gtk_table_attach (GTK_TABLE (table4), checkbutton13, 0, 1, 0, 1,
		(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton13), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton13), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton13), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  checkbutton19 = gtk_check_button_new();
  gtk_widget_show (checkbutton19);
	gtk_container_add (GTK_CONTAINER (checkbutton19), pixmap9);
  gtk_table_attach (GTK_TABLE (table4), checkbutton19, 1, 2, 0, 1,
                    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton19), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton19), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton19), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  checkbutton14 = gtk_check_button_new();
  gtk_widget_show (checkbutton14);
	gtk_container_add (GTK_CONTAINER (checkbutton14), pixmap4);
  gtk_table_attach (GTK_TABLE (table4), checkbutton14, 2, 3, 0, 1,
                    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton14), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton14), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton14), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  checkbutton15 = gtk_check_button_new();
  gtk_widget_show (checkbutton15);
	gtk_container_add (GTK_CONTAINER (checkbutton15), pixmap5);
  gtk_table_attach (GTK_TABLE (table4), checkbutton15, 3, 4, 0, 1,
                    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton15), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton15), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton15), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  checkbutton16 = gtk_check_button_new();
  gtk_widget_show (checkbutton16);
	gtk_container_add (GTK_CONTAINER (checkbutton16), pixmap6);
  gtk_table_attach (GTK_TABLE (table4), checkbutton16, 0, 1, 1, 2,
                    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton16), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton16), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton16), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  checkbutton20 = gtk_check_button_new();
  gtk_widget_show (checkbutton20);
	gtk_container_add (GTK_CONTAINER (checkbutton20), pixmap10);
  gtk_table_attach (GTK_TABLE (table4), checkbutton20, 1, 2, 1, 2,
                    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton20), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton20), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton20), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  checkbutton17 = gtk_check_button_new();
  gtk_widget_show (checkbutton17);
	gtk_container_add (GTK_CONTAINER (checkbutton17), pixmap7);
  gtk_table_attach (GTK_TABLE (table4), checkbutton17, 2, 3, 1, 2,
                    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton17), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton17), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton17), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  checkbutton18 = gtk_check_button_new();
  gtk_widget_show (checkbutton18);
	gtk_container_add (GTK_CONTAINER (checkbutton18), pixmap8);
  gtk_table_attach (GTK_TABLE (table4), checkbutton18, 3, 4, 1, 2,
                    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton18), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (checkbutton18), TRUE);
	gtk_signal_connect (GTK_OBJECT (checkbutton18), "clicked",
		GTK_SIGNAL_FUNC(checkbox_cb), this);

  frame7 = gtk_frame_new ("Date");
  gtk_widget_show (frame7);
  gtk_box_pack_start (GTK_BOX (hbox32), frame7, FALSE, TRUE, 0);

  vbox28 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox28);
  gtk_container_add (GTK_CONTAINER (frame7), vbox28);
  gtk_container_set_border_width (GTK_CONTAINER (vbox28), 4);

  spinbutton4_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  spinbutton4 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton4_adj), 1, 0);
  gtk_widget_show (spinbutton4);
  gtk_box_pack_start (GTK_BOX (vbox28), spinbutton4, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (spinbutton4_adj), "value_changed",
		GTK_SIGNAL_FUNC (spinner_cb), this);

	hscrollbar2_adj = gtk_adjustment_new (1, 1, 100, 1, 10, 10);
  hscrollbar2 = gtk_hscrollbar_new (GTK_ADJUSTMENT (hscrollbar2_adj));
  gtk_widget_show (hscrollbar2);
  gtk_box_pack_start (GTK_BOX (vbox28), hscrollbar2, FALSE, TRUE, 0);
  gtk_widget_set_usize (hscrollbar2, 60, -2);
	gtk_signal_connect (GTK_OBJECT (hscrollbar2_adj), "value_changed",
		GTK_SIGNAL_FUNC (scroll_cb), this);

  hbox46 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox46);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox46, FALSE, TRUE, 0);

  button39 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button39);
  gtk_box_pack_end (GTK_BOX (hbox46), button39, FALSE, FALSE, 0);
  gtk_widget_set_usize (button39, 100, -2);
  gtk_signal_connect (GTK_OBJECT (button39), "clicked",
		GTK_SIGNAL_FUNC (on_Close_activate), this);

	gtk_widget_realize(drawingarea1);

	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
		(GtkSignalFunc) expose_event, this);
	gtk_signal_connect (GTK_OBJECT(drawingarea1),"configure_event",
		(GtkSignalFunc) configure_event, this);
}

void Graph::show()
{
	gtk_widget_show(window1);
	gdk_window_raise(window1->window);
	load();
}

void Graph::hide()
{
	gtk_widget_hide(window1);
}

void Graph::init()
{
	GtkAdjustment *adj;
	Router *r;
	Link *ln;

	length = 1;
	adj = GTK_ADJUSTMENT(spinbutton4_adj);
	adj->value = 1; adj->lower = 1; adj->upper = 28;
	adj->step_increment = 1; adj->page_increment = 10; adj->page_size = 10;

	start_graph = 0;
	adj = GTK_ADJUSTMENT(hscrollbar2_adj);
	adj->step_increment = 1; adj->page_increment = 10; adj->page_size = 1;
	adj->value = 0; adj->lower = -640; adj->upper = 0+adj->page_size;

	if (main->files->objects==NULL) return;

	main->files->objects->routers.sort(compare_router_names, BUBBLE_SORT);

	combo1_items = NULL;
	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		combo1_items = g_list_append(combo1_items, r->name);
	}
  gtk_combo_set_popdown_strings(GTK_COMBO(combo3), combo1_items);
  g_list_free (combo1_items);
}

void Graph::router()
{
	GList *combo_items = NULL;
	char *string;
	char buf[256];
	Link *ln, *ln2;
	Router *r, *dv=NULL;
	Interface *in;

	string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo3)->entry));

	// find the router we want interfaces for
	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if (strcmp(r->name, string)==0) dv = r;
	}

	if (dv != NULL)
	{
		if (dv->interfaces.size()<1) return;
		for (ln2=dv->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			sprintf(buf, "%s", in->port);
			if (strcmp(buf, "")!=0)
				combo_items = g_list_append(combo_items, strdup(buf));
		}
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(combo4), combo_items);
	g_list_free(combo_items);
}

void Graph::spinner()
{
	GtkAdjustment *adj = (GtkAdjustment*)spinbutton4_adj;
	// check start/end data values (if we leave the program running all the
	// time we will need to update these variable once we go past midnight)

	length = (gint) adj->value;
	draw();
}

void Graph::scroll()
{
	GtkAdjustment *adj = (GtkAdjustment*)hscrollbar2_adj;

	start_graph = (gint) adj->value;
	draw();
}

void Graph::load()
{
	FILE *f;
	struct stat statbuf;
	char buf[256], *router, *index;
	int i, flag, count;
	unsigned long offset;
	Link *ln;
	Router *r;
	Interface *in;

	router = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo3)->entry));
	index = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo4)->entry));

	current_router = NULL;
	current_interface = NULL;

	if (main->files->objects==NULL) return;

	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if (strcmp(r->name, router)==0)
			current_router = r;
	}
	if (current_router==NULL) return;

	for (ln=current_router->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		sprintf(buf, "%s", in->port);
		if (strcmp(buf, index)==0)
			current_interface = in;
	}
	if (current_interface==NULL) return;

	sprintf(buf, "%s/graph/%s/%d", main->pref->server_directory, current_router->name, current_interface->index);
	if (strcmp(current_file, buf)==0) return;
	strcpy(current_file, buf);

	size = 0;

	// look at the size of the file and work out how big the array
	// of data needs to be

	// (8*4)+8 = 40 bytes per time
	if (stat(current_file, &statbuf) == -1) return;
	size = (statbuf.st_size-12)/40;

	if (iface != NULL)
		delete [] iface;
	iface = new Iface[size];

	f = fopen(current_file, "r");
	if (!f) return;

	fseek(f, 8L, SEEK_SET);
	offset = read_ulong(f);
	fseek(f, offset, SEEK_SET);

	iface[0].timestamp = read_time_t(f);
	if (iface[0].timestamp == 0) offset = 12;
	fseek(f, offset, SEEK_SET);

	// read timestamp at offset,
	// if 0 then real start is at the beginning of the file
	// else real start is at offset

	count = 0;
	flag = 0;
	while ((count<size) && (flag==0))
	{
		iface[count].timestamp = read_time_t(f);
		if (iface[count].timestamp!=0)
		{
			for (i=0; i<4; i++) fgetc(f);
			iface[count].octin = read_ulong(f);
			iface[count].octout = read_ulong(f);
			iface[count].bcastin = read_ulong(f);
			iface[count].bcastout = read_ulong(f);
			iface[count].disin = read_ulong(f);
			iface[count].disout = read_ulong(f);
			iface[count].errin = read_ulong(f);
			iface[count].errout = read_ulong(f);
			count++;
			if (offset+(count*40) >= ((unsigned long)(statbuf.st_size)))
			{
				offset = 12;
				fseek(f, offset, SEEK_SET);
			}
		}else
		{
			flag = 1;
		}
	}

	size = count;

/*
	struct tm *t;
	time_t ts;
	printf("Version #\n");
	printf("Size %d\n", size);
	printf("Offset %lu\n", offset);
	for (i=0; i<size; i++)
	{
		ts = iface[i].timestamp;
		t = gmtime(&ts);

		printf("  Timestamp %s", asctime(t)); 
		printf("  Data %lu %lu %lu %lu %lu %lu %lu %lu\n", iface[i].octin, iface[i].octout, iface[i].bcastin, iface[i].bcastout, iface[i].disin, iface[i].disout, iface[i].errin, iface[i].errout);
	}
*/

	draw();
	fclose(f);
}




void Graph::draw()
{
	gint ow, oh;
	char buf[256], buf2[256], buf3[256];
	time_t tmr;
	struct tm *tb;
	int i, j, txtw, num_speeds, num_times, first_day, day_len, type;
	gint8 dash_list[2];

	oh = height;
	ow = width;

	if (current_router==NULL) return;
	if (current_interface==NULL) return;
	if (drawingarea1->window == NULL) return;

	height = drawingarea1->allocation.height;
	width = drawingarea1->allocation.width;

/*
	if (height < viewport1->allocation.height)
		height = viewport1->allocation.height;
	if (width < viewport1->allocation.width)
		width = viewport1->allocation.width;
*/

	if ((canvas == NULL) || (oh != height) || (ow != width))
	{
		if (canvas) gdk_pixmap_unref(canvas);
		canvas = gdk_pixmap_new(drawingarea1->window, width, height, -1);
	}

// work out times
	tmr = time(NULL);
	tb = gmtime(&tmr);
	tb->tm_sec = 0;
	tb->tm_min = 0;
	tb->tm_hour = 0;
	tmr = mktime(tb);
	// midnight lastnight
	first_day = tb->tm_wday;

	gstart = tmr + (86400*start_graph);
	gend = gstart + (86400*length);

	SX = BX = LEFT_MGN;
	SW = BW = width-LEFT_MGN-RIGHT_MGN;
	SH = (height-TOP_MGN-GAP-BOTTOM_MGN)/4;
	if (SH<40) SH=40;
	if (SH>80) SH=80;
	BH = height-TOP_MGN-GAP-BOTTOM_MGN-SH;
	SY = TOP_MGN;
	BY = TOP_MGN+SH+GAP;

// clear window
	int x, y, w, h;
	GtkStyle *style;

	x = drawingarea1->allocation.width;
	y = drawingarea1->allocation.height;
	w = x/128;
	h = y/128;
	style = gtk_widget_get_style(window1);
	for (x=0; x<=w; x++)
		for (y=0; y<=h; y++)
			gtk_draw_flat_box(style, canvas, GTK_STATE_NORMAL,
				GTK_SHADOW_NONE, x*128, y*128, 128, 128);

// speeds
	gdk_gc_set_foreground(gc, &main->black);
	num_speeds = (BH/20)-1;		// 20 is room for text + a gap
	// 1,2,4,5,10,20,25
	if (num_speeds==3) num_speeds=2; else
	if ((num_speeds>=6) && (num_speeds<=9)) num_speeds=5; else
	if ((num_speeds>=11) && (num_speeds<=19)) num_speeds=10; else
	if ((num_speeds>=21) && (num_speeds<=24)) num_speeds=20; else
	if (num_speeds>=26) num_speeds=25;
	for (i=0; i<num_speeds+1; i++)
	{
		speed_to_str(current_interface->speed*i/num_speeds, buf);
		txtw = gdk_string_width(main->font10m, buf);
		gdk_draw_string(canvas, main->font10m, gc, BX-2-txtw, BY+BH-(BH*i/num_speeds)+5, buf);
		gdk_draw_line(canvas, gc, BX-2, BY+BH-(BH*i/num_speeds), BX, BY+BH-(BH*i/num_speeds));
	}

// draw times

	num_times = ((BW/length)/14)-1;		// 18 is room for text + a gap

	// 1,2,3,4,6,8,12,24
	if (num_times<=0) num_times=1; else
	if (num_times==5) num_times=4; else
	if (num_times==7) num_times=6; else
	if ((num_times>=9) && (num_times<=11)) num_times=8; else
	if ((num_times>=13) && (num_times<=23)) num_times=12; else
	if (num_times>=25) num_times=24;

	for (i=0; i<(num_times*length)+1; i++)
	{
		sprintf(buf, "%d", ((24/num_times)*i)%24);
		txtw = gdk_string_width(main->font10m, buf);
		gdk_draw_string(canvas, main->font10m, gc,
			BX-(txtw/2)+(BW*i/(num_times*length)), height+12-BOTTOM_MGN, buf);
		gdk_draw_line(canvas, gc, BX+(BW*i/(num_times*length)), height-BOTTOM_MGN,
			BX+(BW*i/(num_times*length)), height-BOTTOM_MGN+2);
	}

// start & finish dates
	tb = gmtime(&gstart);
	date_to_str(tb, buf2, 2);
	gtmp = gend - 86400;
	tb = gmtime(&gtmp);
	date_to_str(tb, buf3, 2);

// a long date to work out string lengths
	tb = gmtime(&tmr);
	tb->tm_mon = 4; // May
	tb->tm_mday = 24;
	tb->tm_year = 100; // 2000
	tb->tm_wday = 3; // Wed

// work out which format to use from the string lengths
	type = 6;
	day_len = (BW/length);
	for (i=5; i>1; i--)
	{
		date_to_str(tb, buf, i);
		if (gdk_string_width(main->font10m, buf)+4<day_len) type=i;
	}

// print the days
	for (i=0; i<length; i++)
	{
		gtmp = gstart + (86400*i);
		tb = gmtime(&gtmp);
		date_to_str(tb, buf, type);
		txtw = gdk_string_width(main->font10m, buf);
		gdk_draw_string(canvas, main->font10m, gc, BX+((day_len-txtw)/2)+(BW*i/length), height+24-BOTTOM_MGN, buf);
	}

// set window title with full dates
	if (length>1)
		sprintf(buf, "Network Graph - %s -> %s", buf2, buf3);
	else
		sprintf(buf, "Network Graph - %s", buf2);
	gtk_window_set_title (GTK_WINDOW (window1), buf);

// draw grid
	gdk_gc_set_foreground(gridgc, &main->lightgray);
	for (i=0; i<(num_times*length); i++)
	{
		if (i%num_times==0)
		{
			j = (first_day+start_graph+1+(i/num_times));
			while (j<0) j+=777;
			if ((j%7==0) || (j%7==1))
			{
				gdk_draw_rectangle(canvas, gridgc, TRUE,
					BX+(BW*i/(num_times*length)), BY, (BW/length)+1, BH);
				gdk_draw_rectangle(canvas, gridgc, TRUE,
					SX+(SW*i/(num_times*length)), SY, (SW/length)+1, SH);
			}
		}
	}

	gdk_gc_set_foreground(gridgc, &main->gray);
	gdk_gc_set_line_attributes (gridgc, 1, GDK_LINE_ON_OFF_DASH,
		GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
	dash_list[0] = 2; dash_list[1] = 2;
	gdk_gc_set_dashes(gridgc, 0, dash_list, sizeof(dash_list));
	for (i=1; i<(num_times*length); i++)
	{
		gdk_draw_line(canvas, gridgc, BX+(BW*i/(num_times*length)), BY+BH,
			BX+(BW*i/(num_times*length)), BY);
		gdk_draw_line(canvas, gridgc, SX+(SW*i/(num_times*length)), SY+SH,
			SX+(SW*i/(num_times*length)), SY);
	}
	for (i=1; i<num_speeds; i++)
	{
		gdk_draw_line(canvas, gridgc, BX, BY+BH-(BH*i/num_speeds), BX+BW,
			BY+BH-(BH*i/num_speeds));
	}

	gdk_gc_set_foreground(gridgc, &main->darkgray);
	dash_list[0] = 4; dash_list[1] = 4;
	gdk_gc_set_dashes(gridgc, 0, dash_list, sizeof(dash_list));
	for (i=0; i<(num_times*length); i++)
	{
		if (i%num_times==0)
		{
			gdk_draw_line(canvas, gridgc, BX+(BW*i/(num_times*length)),
				BY+BH, BX+(BW*i/(num_times*length)), BY);
			gdk_draw_line(canvas, gridgc, SX+(SW*i/(num_times*length)),
				SY+SH, SX+(SW*i/(num_times*length)), SY);
		}
	}

// draw graph
	if (GTK_TOGGLE_BUTTON (checkbutton13)->active) draw_line(1, &main->red, current_interface->speed/8);
	if (GTK_TOGGLE_BUTTON (checkbutton16)->active) draw_line(2, &main->darkgreen, current_interface->speed/8);
	if (GTK_TOGGLE_BUTTON (checkbutton19)->active) draw_line(3, &main->blue, 20);
	if (GTK_TOGGLE_BUTTON (checkbutton20)->active) draw_line(4, &main->orange, 20);
	if (GTK_TOGGLE_BUTTON (checkbutton14)->active) draw_line(5, &main->darkgray, 20);
	if (GTK_TOGGLE_BUTTON (checkbutton17)->active) draw_line(6, &main->magenta, 20);
	if (GTK_TOGGLE_BUTTON (checkbutton15)->active) draw_line(7, &main->purple, 20);
	if (GTK_TOGGLE_BUTTON (checkbutton18)->active) draw_line(8, &main->darkred, 20);

// draw axis
	gdk_gc_set_foreground(gc, &main->black);
	gdk_draw_rectangle(canvas, gc, FALSE, BX, BY, BW, BH);
	gdk_draw_rectangle(canvas, gc, FALSE, SX, SY, SW, SH);

// force an expose event
	gdk_draw_pixmap(drawingarea1->window,
		drawingarea1->style->fg_gc[GTK_WIDGET_STATE(drawingarea1)],
		canvas, 0, 0, 0, 0, drawingarea1->allocation.width,
		drawingarea1->allocation.height);
}

void Graph::draw_line(int l, GdkColor *col, int scale)
{
	unsigned long val1, val2;
	gint i, x1, y1, x2, y2;
	int x, y, w, h;
	GdkRectangle rect;

	if (current_interface==NULL) return;
	if (iface==NULL) return;
	if (current_interface->speed==0) return;

	switch(l)
	{
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
		case 8: x=SX; y=SY; w=SW; h=SH; break;
		case 1:
		case 2:
		default: x=BX; y=BY; w=BW; h=BH;
	}

// set clip region for gc
	rect.x = x; rect.y = y;
	rect.width = w; rect.height = h;
	gdk_gc_set_clip_rectangle(gc, &rect);

	gtmp2 = iface[0].timestamp;
	gdk_gc_set_foreground(gc, col);

	for (i=1; i<size; i++)
	{
		if (l==1) { val1 = iface[i-1].octin; val2 = iface[i].octin; } else
		if (l==2) { val1 = iface[i-1].octout; val2 = iface[i].octout; } else
		if (l==3) { val1 = iface[i-1].bcastin; val2 = iface[i].bcastin; } else
		if (l==4) { val1 = iface[i-1].bcastout; val2 = iface[i].bcastout; } else
		if (l==5) { val1 = iface[i-1].disin; val2 = iface[i].disin; } else
		if (l==6) { val1 = iface[i-1].disout; val2 = iface[i].disout; } else
		if (l==7) { val1 = iface[i-1].errin; val2 = iface[i].errin; } else
		if (l==8) { val1 = iface[i-1].errout; val2 = iface[i].errout; } else
		{ val1 = val2 = 0; }
		gtmp = iface[i].timestamp;
		gtmp2 = iface[i-1].timestamp;
		if ((gtmp>gstart) && (gtmp<gend))
		{
			x1 = ((gtmp2-gstart)*w/(gend-gstart))+x;
			x2 = ((gtmp-gstart)*w/(gend-gstart))+x;
			y1 = y+h-(val1*h/scale);
			y2 = y+h-(val2*h/scale);
//				unsigned long t1;
//				t1 = (unsigned long) scale;
//				y1 = y+h-(((unsigned long)(val1/t1))*h);
//				y2 = y+h-(((unsigned long)(val2/t1))*h);
			gdk_draw_line(canvas, gc, x1, y1, x2, y2);
		}
		gtmp2 = gtmp;
	}

// release clip region for gc, i.e. set it to full size
	gdk_gc_set_clip_rectangle(gc, NULL);
}

