/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "misc.h"
#include "images/icon.xpm"

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((Help*)data)->hide();
}

static void on_Back_activate(GtkWidget *w, gpointer data)
{
	((Help*)data)->back_cb();
}

void Help::back_cb()
{
	if (current_go>1)
	{
		current_go--;
		strcpy(help_file, go_list[current_go-1]);
	}
	draw();
}

static void on_Forward_activate(GtkWidget *w, gpointer data)
{
	((Help*)data)->forward_cb();
}

void Help::forward_cb()
{
	if (current_go<num_go)
	{
		current_go++;
		strcpy(help_file, go_list[current_go-1]);
	}
	draw();
}

static void on_Home_activate(GtkWidget *w, gpointer data)
{
	((Help*)data)->home_cb();
}

void Help::home_cb()
{
	if (current_go<MAX_GO)
	{
		strcpy(go_list[current_go], "index.html");
		current_go++;
		num_go = current_go;
	}
	draw();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Help*)data)->hide();
	return (TRUE); 
}

static gint expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	Help *m = (Help*)data;

	if (m->canvas)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			m->canvas, event->area.x, event->area.y, event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	return FALSE;
}

static gint configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer data)
{
	// Hmm the window has been resized so there will be an expose event along
	// in a minute to copy the canvas to the screen, but we need to redraw
	// the canvas as the size could have grown to fit the window
	((Help*)data)->draw();
	return TRUE;
}

static gint button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	((Help*)data)->check_link((int) event->x, (int) event->y);
	return TRUE;
}

static gint motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
	((Help*)data)->move_link((int) event->x, (int) event->y);
	return TRUE;
}

Help::Help(Main *m)
{
	struct passwd *p;
	char mapdir[256];

	p = getpwuid(getuid());
	sprintf(mapdir, "%s/%s", p->pw_dir, HOME_DIR);
	sprintf(help_dir, "%s/help", mapdir);
	sprintf(help_file, "index.html");
	num_go = current_go = 1;
	strcpy(go_list[0], "index.html");
	num_hlink = 0;

	main = m;
	canvas = NULL;
	width = height = 0;

	hand_cursor = gdk_cursor_new(GDK_HAND2);
	arrow_cursor = gdk_cursor_new(GDK_TOP_LEFT_ARROW);

	create();
	gc  = gdk_gc_new(window1->window);
  gdk_gc_set_line_attributes(gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT,
		GDK_JOIN_MITER);
	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar1), "URL");
}

Help::~Help()
{
	if (gc) gdk_gc_unref(gc);
}

void Help::create()
{
  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize (window1, 480, 480);
  gtk_window_set_title (GTK_WINDOW (window1), "Network Monitor - Help");
  gtk_window_set_policy (GTK_WINDOW (window1), TRUE, TRUE, FALSE);
	gtk_widget_realize (window1);
	window_icon(window1, icon_xpm);
	gdk_window_set_icon_name(window1->window, "Help");
	gtk_signal_connect(GTK_OBJECT(window1), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

	vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, TRUE, 0);

  File = gtk_menu_item_new_with_label ("File");
  gtk_widget_show (File);
  gtk_container_add (GTK_CONTAINER (menubar1), File);

  menu2 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (File), menu2);

  Close = gtk_menu_item_new_with_label ("Close");
  gtk_widget_show (Close);
  gtk_container_add (GTK_CONTAINER (menu2), Close);
  gtk_signal_connect (GTK_OBJECT (Close), "activate", GTK_SIGNAL_FUNC (on_Close_activate), this);

  Home = gtk_menu_item_new_with_label ("Home");
  gtk_widget_show (Home);
  gtk_container_add (GTK_CONTAINER (menubar1), Home);
  gtk_signal_connect (GTK_OBJECT (Home), "activate", GTK_SIGNAL_FUNC (on_Home_activate), this);

  Back = gtk_menu_item_new_with_label ("Back");
  gtk_widget_show (Back);
  gtk_container_add (GTK_CONTAINER (menubar1), Back);
  gtk_signal_connect (GTK_OBJECT (Back), "activate", GTK_SIGNAL_FUNC (on_Back_activate), this);

  Forward = gtk_menu_item_new_with_label ("Forward");
  gtk_widget_show (Forward);
  gtk_container_add (GTK_CONTAINER (menubar1), Forward);
  gtk_signal_connect (GTK_OBJECT (Forward), "activate", GTK_SIGNAL_FUNC (on_Forward_activate), this);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  drawingarea1 = gtk_drawing_area_new();
	gtk_widget_set_events (drawingarea1, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK);
  gtk_widget_show(drawingarea1);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwindow1),
		drawingarea1);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 0);

	gtk_widget_realize(drawingarea1);

	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
		(GtkSignalFunc) expose_event, this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1),"configure_event",
		(GtkSignalFunc) configure_event, this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "button_press_event",
		(GtkSignalFunc) button_press_event, this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "motion_notify_event",
		(GtkSignalFunc) motion_notify_event, this);
}

void Help::show()
{
	gtk_widget_show(window1);
	gdk_window_raise(window1->window);
}

void Help::hide()
{
	gtk_widget_hide(window1);
}

void Help::draw()
{
	int oh, ow;

	if (drawingarea1->window == NULL) return;

	oh = height;
	ow = width;

	width = drawingarea1->allocation.width;
	height = drawingarea1->allocation.height;
	if (height<100) height = 100;
	if (width<100) width = 100;

	if ((canvas == NULL) || (oh != height) || (ow != width))
	{
// printf("CREATING CANVAS %d %d\n", width, height);
		if (canvas) gdk_pixmap_unref(canvas);
	  gtk_widget_set_usize (drawingarea1, width, height);
		canvas = gdk_pixmap_new(drawingarea1->window, width, height, -1);
	}

	load();

	// force an expose event
	gdk_draw_pixmap(drawingarea1->window,
		drawingarea1->style->fg_gc[GTK_WIDGET_STATE(drawingarea1)],
		canvas, 0, 0, 0, 0, drawingarea1->allocation.width,
		drawingarea1->allocation.height);
}

void Help::load()
{
	FILE *f;
//	GdkPixmap *pixmap;
//	GdkImlibImage *im;
	GdkColor *col;
	char token[256];
	char fname[256];
	char buf[1024];
//	char b1[256];
	int ch, status, token_pos, buf_pos;
	gint x, y;
//	gint w, h;

	if (canvas==NULL) return;

	sprintf(buf, "%s/%s", help_dir, help_file);
// printf("HELP %s\n", buf);
	f = fopen(buf, "r");
	if (!f) return;

	// clear window
	gdk_gc_set_foreground(gc, &main->white);
	gdk_draw_rectangle(canvas, gc, TRUE, 0, 0, width, height);
	col = &main->black;

	gtk_statusbar_pop(GTK_STATUSBAR(statusbar1), context_id);

	x = y = 2;
	ch = 0;
	status = 0;
	buf_pos = 0;
	num_hlink = 0;
	while ((ch=fgetc(f))!=EOF)
	{
		if (ch == '\n')
		{
		}else
		if (ch == '<')
		{
			if (buf_pos>0)
			{
				buf[buf_pos] = '\0';
				buf_pos = 0;
			}
			status = 1;
			token[0] = ch;
			token_pos = 1;
		}else
		if (ch == '>')
		{
			status = 0;
			token[token_pos] = ch;
			token_pos++;
			token[token_pos] = '\0';

			// parse tokens
			if (strcmp(token, "<h1>")==0)
			{
				y += 18;
				x = 2;
				gdk_gc_set_foreground(gc, col);
			}else
			if (strcmp(token, "</h1>")==0)
			{
				gdk_draw_text(canvas, main->font18b, gc, x, y, buf, strlen(buf));
				gdk_gc_set_foreground(gc, col);
				buf[0] = '\0';
				x = 2;
				y += 12;
			}else
			if (strcmp(token, "<p>")==0)
			{
				gdk_draw_text(canvas, main->font12m, gc, x, y+12, buf, strlen(buf));
				buf[0] = '\0';
				y += 12;
				y += 12;
				x = 2;
			}else
			if (strcmp(token, "<br>")==0)
			{
				gdk_draw_text(canvas, main->font12m, gc, x, y+12, buf, strlen(buf));
				buf[0] = '\0';
				y += 12;
				x = 2;
			}else
			if (strcmp(token, "</a>")==0)
			{
				gdk_gc_set_foreground(gc, &main->blue);
				gdk_draw_text(canvas, main->font12m, gc, x, y+12, buf, strlen(buf));
				gdk_draw_line(canvas, gc, x, y+13, x+gdk_text_width(main->font12m, buf, strlen(buf)), y+13);	

				if (num_hlink<MAX_HLINK)
				{
					link[num_hlink].x = x;
					link[num_hlink].y = y+12;
					link[num_hlink].w = gdk_text_width(main->font12m, buf, strlen(buf));
					link[num_hlink].h = 12;
					strcpy(link[num_hlink].fname, fname);
					num_hlink++;
				}

				x += gdk_text_width(main->font12m, buf, strlen(buf))+2;
				buf[0] = '\0';
				gdk_gc_set_foreground(gc, &main->black);
			}else
			if (sscanf(token, "<img src=\"%[^\"]\">", fname)==1)
			{
/*
				sprintf(b1, "%s/%s", help_dir, fname);
				im = gdk_imlib_load_image(b1);
				pixmap = load_pixmap(drawingarea1, b1);
				if (im != NULL)
				{
					w = im->rgb_width;
					h = im->rgb_height;
					gdk_imlib_render(im, w, h);
					pixmap = gdk_imlib_move_image(im);
					gdk_draw_pixmap(canvas, gc, pixmap, 0, 0, x, y, w, h);
					gdk_imlib_free_pixmap(pixmap);
					x += w+2;
					y += h+2;
				}
*/
			}else
			if (sscanf(token, "<a href=\"%[^\"]\">", fname)==1)
			{
				gdk_draw_text(canvas, main->font12m, gc, x, y+12, buf, strlen(buf));
				x += gdk_text_width(main->font12m, buf, strlen(buf))+2;
				buf[0] = '\0';
			}else
			if (sscanf(token, "<font color=%[^>]>", fname)==1)
			{
				gdk_draw_text(canvas, main->font12m, gc, x, y+12, buf, strlen(buf));
				x += gdk_text_width(main->font12m, buf, strlen(buf))+2;
				buf[0] = '\0';
				if (strcmp(fname, "red")==0) col = &main->red; else
				if (strcmp(fname, "green")==0) col = &main->green; else
				if (strcmp(fname, "yellow")==0) col = &main->yellow; else
				if (strcmp(fname, "blue")==0) col = &main->blue; else
				if (strcmp(fname, "orange")==0) col = &main->orange; else
				if (strcmp(fname, "darkred")==0) col = &main->darkred; else
				if (strcmp(fname, "cyan")==0) col = &main->cyan; else
				if (strcmp(fname, "magenta")==0) col = &main->magenta; else
				if (strcmp(fname, "purple")==0) col = &main->purple; else
				if (strcmp(fname, "white")==0) col = &main->white; else
				if (strcmp(fname, "lightgray")==0) col = &main->lightgray; else
				if (strcmp(fname, "gray")==0) col = &main->gray; else
				if (strcmp(fname, "darkgreen")==0) col = &main->darkgreen; else
					col = &main->black;
				gdk_gc_set_foreground(gc, col);
			}else
			if (strcmp(token, "</font>")==0)
			{
				gdk_draw_text(canvas, main->font12m, gc, x, y+12, buf, strlen(buf));
				x += gdk_text_width(main->font12m, buf, strlen(buf))+2;
				buf[0] = '\0';
				col = &main->black;
				gdk_gc_set_foreground(gc, col);
			}else
			{
				gdk_draw_text(canvas, main->font12m, gc, x, y+12, buf, strlen(buf));
				x += gdk_text_width(main->font12m, buf, strlen(buf))+2;
				buf[0] = '\0';
			}
		}else
		if (status == 1)
		{
			token[token_pos] = ch;
			token_pos++;
		}else
		{
			buf[buf_pos] = ch;
			buf_pos++;
		}
	}
	fclose(f);
}

void Help::check_link(int x, int y)
{
	int i;

	for (i=0; i<num_hlink; i++)
	{
		if ((x>=link[i].x) && (x<=link[i].x+link[i].w) &&
				(y>=link[i].y-link[i].h) && (y<=link[i].y))
		{
			strcpy(help_file, link[i].fname);
			if (current_go<MAX_GO)
			{
				strcpy(go_list[current_go], link[i].fname);
				current_go++;
				num_go = current_go;
			}
			draw();
			break;
		}
	}
}

void Help::move_link(int x, int y)
{
	GdkCursor *cur = arrow_cursor;
	int i;

	gtk_statusbar_pop(GTK_STATUSBAR(statusbar1), context_id);

	for (i=0; i<num_hlink; i++)
	{
		if ((x>=link[i].x) && (x<=link[i].x+link[i].w) &&
				(y>=link[i].y-link[i].h) && (y<=link[i].y))
		{
			cur = hand_cursor;
			gtk_statusbar_push(GTK_STATUSBAR(statusbar1), context_id, link[i].fname);
		}
	}

	gdk_window_set_cursor(drawingarea1->window, cur);
}

