/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "main.h"
#include "misc.h"
#include "subnet.h"
#include "network.h"
#include "router.h"
#include "machine.h"
#include "interface.h"
#include "ipaddress.h"
#include "images/network.xpm"
#include "images/icon.xpm"

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((MachineWindow*)data)->hide();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((MachineWindow*)data)->hide();
	return (TRUE); 
}

void MachineWindow::tags()
{
	main->files->clear_tags();
	if (current_machine != NULL)
		if (current_machine->network != NULL)
			current_machine->network->selected = 1;
	main->monitor->draw();
}

static void machine_list_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	MachineWindow *m = (MachineWindow*) data;
	gchar *text;
	unsigned char mc[6];

	gtk_clist_get_text(GTK_CLIST(clist), row, 1, &text);	// MAC address
	convert_mac_3(text, mc);
	m->select_interface(mc);
	m->tags();
}

static void machine_list_cb2(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	MachineWindow *m = (MachineWindow*) data;
	m->current_machine = NULL;
	m->expose_machine();
	m->tags();
}

static void resize_cb(GtkCList *clist, gint column, gint width, gpointer data)
{
	MachineWindow *m = (MachineWindow*) data;
	gtk_clist_set_column_width(GTK_CLIST(m->clist20), column, width);
}

static gint clist_compare_ip(GtkCList *cl, gconstpointer p1, gconstpointer p2)
{
	GtkCListRow *row;
	Machine *sn, *s2;

	row = (GtkCListRow*) p1;
	sn = (Machine*) row->data;
	row = (GtkCListRow*) p2;
	s2 = (Machine*) row->data;

	return sn->ip.real_compare(s2->ip);
}

static void title_cb(GtkWidget *clist, gint column, gpointer data)
{
	MachineWindow *snw = (MachineWindow*) data;
	snw->current_sort = column;

	gtk_clist_set_sort_column (GTK_CLIST (clist), column);

	if (column==2)
	{
		gtk_clist_set_compare_func (GTK_CLIST (clist), clist_compare_ip);
	}else
		gtk_clist_set_compare_func (GTK_CLIST (clist), NULL);

	gtk_clist_sort (GTK_CLIST (clist));
	snw->machine_string(snw->current_machine->name);
}

static void check_cb(GtkWidget *w, gpointer data)
{
	MachineWindow *m = (MachineWindow*) data;
	m->init();
}

static gint expose_event(GtkWidget *w, GdkEventExpose *event, gpointer data)
{ ((MachineWindow*)data)->expose_machine(); return FALSE; }

MachineWindow::MachineWindow(Main *m)
{
	main = m;
	current_sort = 0;
	current_machine = NULL;
	machine_list = NULL;
	create();
	gc = gdk_gc_new(machine->window);
}

MachineWindow::~MachineWindow()
{
}

void MachineWindow::create()
{
  machine = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (machine), "Machine Details");
  gtk_widget_realize (machine);
	window_icon(machine, icon_xpm);
	gdk_window_set_icon_name(machine->window, "Machine");
	gtk_signal_connect(GTK_OBJECT(machine), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  vbox1 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (machine), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 4);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_widget_set_usize (scrolledwindow1, -2, 100);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist1 = gtk_clist_new (3);
  gtk_widget_show (clist1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), clist1);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 0, 120);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 1, 97);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 2, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist1));
	gtk_signal_connect(GTK_OBJECT(clist1), "select_row",
		GTK_SIGNAL_FUNC(machine_list_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist1), "unselect_row",
		GTK_SIGNAL_FUNC(machine_list_cb2), this);
	gtk_signal_connect(GTK_OBJECT(clist1), "resize_column",
		GTK_SIGNAL_FUNC(resize_cb), this);
  gtk_signal_connect (GTK_OBJECT (clist1), "click_column",
		GTK_SIGNAL_FUNC (title_cb), this);

  label4 = gtk_label_new ("Name");
  gtk_widget_show (label4);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 0, label4);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

  label5 = gtk_label_new ("MAC Address");
  gtk_widget_show (label5);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 1, label5);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  label6 = gtk_label_new ("Interface");
  gtk_widget_show (label6);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 2, label6);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

  scrolledwindow21 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow21);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow21, FALSE, TRUE, 0);
  gtk_widget_set_usize (scrolledwindow21, -2, 40);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow21), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist20 = gtk_clist_new (3);
  gtk_widget_show (clist20);
  gtk_container_add (GTK_CONTAINER (scrolledwindow21), clist20);
  gtk_clist_set_column_width (GTK_CLIST (clist20), 0, 120);
  gtk_clist_set_column_width (GTK_CLIST (clist20), 1, 97);
  gtk_clist_set_column_width (GTK_CLIST (clist20), 2, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (clist20));

	drawingarea1 = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea1);
  gtk_box_pack_start (GTK_BOX (vbox1), drawingarea1, FALSE, TRUE, 0);
  gtk_widget_set_usize (drawingarea1, 500, 76);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
		(GtkSignalFunc) expose_event, this);

  hbox45 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox45);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox45, FALSE, TRUE, 0);

  checkbutton39 = gtk_check_button_new_with_label ("All Machines");
  gtk_widget_show (checkbutton39);
  gtk_box_pack_start (GTK_BOX (hbox45), checkbutton39, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (checkbutton39), "clicked",
		(GtkSignalFunc) check_cb, this);

  button43 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button43);
  gtk_box_pack_end (GTK_BOX (hbox45), button43, FALSE, FALSE, 0);
  gtk_widget_set_usize (button43, 100, -2);

  gtk_signal_connect (GTK_OBJECT (button43), "clicked",
		GTK_SIGNAL_FUNC (on_Close_activate), this);
}

void MachineWindow::show()
{
	init();
	gtk_widget_show(machine);
	gdk_window_raise(machine->window);
}

void MachineWindow::hide()
{
	gtk_widget_hide(machine);
}

void MachineWindow::init()
{
	Link *ln;
	gchar *data[3];
	char buf[256];
	Machine *m;
	int row, unreg;
	List *tmp_list;

	if (main->files->objects==NULL) return;
	if (machine_list==NULL) return;

	if (GTK_TOGGLE_BUTTON(checkbutton39)->active) 
		tmp_list = &(main->files->objects->machines);
	else
		tmp_list = machine_list;

	gtk_clist_freeze(GTK_CLIST(clist1));
	gtk_clist_clear(GTK_CLIST(clist1));

	row = 0;
	unreg = 0;
	for (ln=tmp_list->get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		data[0] = strdup(m->name);
		sprint_mac(m->mac, buf);
		data[1] = strdup(buf);
		m->ip.sprint(buf);
		data[2] = strdup(buf);
		gtk_clist_append(GTK_CLIST(clist1), data);
		delete data[0]; delete data[1]; delete data[2];

		if (m->place == 1)
			gtk_clist_set_foreground(GTK_CLIST(clist1), row, &(main->red));
		else
			gtk_clist_set_foreground(GTK_CLIST(clist1), row, &(main->black));
                                    

		gtk_clist_set_row_data(GTK_CLIST(clist1), row, m);
		if (m->registered==0) unreg++;
		row++;
	}

	sprintf(buf, "Name (%d) (%d)", row, unreg);
	gtk_clist_set_column_title(GTK_CLIST(clist1), 0, buf);
	title_cb(clist1, current_sort, this);
	gtk_clist_thaw(GTK_CLIST(clist1));
	current_machine=NULL;
	expose_machine();
}

void MachineWindow::select_interface(unsigned char *str)
{
	Link *ln;
	Machine *m;

	if (main->files->objects==NULL) return;

	for (ln=main->files->objects->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if ((compare_mac(str, m->mac)==1) && (current_machine!=m))
		{
			current_machine = m;
		}
	}
	expose_machine();
}

void MachineWindow::expose_machine()
{
	char buf[256], buf2[256];
	Subnet *sn;
	Link *ln;
	Interface *in;
	Router *rtr;
	gchar *data[3];
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	int row;

	int x, y, w, h;
	GtkStyle *style;

	x = drawingarea1->allocation.width;
	y = drawingarea1->allocation.height;
	w = x/128;
	h = y/128;
	style = gtk_widget_get_style(machine);
	for (x=0; x<=w; x++)
		for (y=0; y<=h; y++)
			gtk_draw_flat_box(style, drawingarea1->window, GTK_STATE_NORMAL,
				GTK_SHADOW_NONE, x*128, y*128, 128, 128);

	gtk_clist_clear(GTK_CLIST(clist20));


	if (current_machine!=NULL)
	{
		gdk_gc_set_foreground(gc, &main->black);
		gdk_draw_text(drawingarea1->window, main->font12b, gc, 2, 12,
		"NAME", 4);
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 102, 12,
			current_machine->name, strlen(current_machine->name));

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 2, 36,
		"IP ADDRESS", 10);
		current_machine->ip.sprint(buf);
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 102, 36,
			buf, strlen(buf));

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 2, 24,
		"MAC ADDRESS", 11);
		sprint_mac(current_machine->mac, buf);
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 102, 24,
			buf, strlen(buf));

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 252, 12,
		"DESCRIPTION", 11);
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 352, 12,
			current_machine->descr, strlen(current_machine->descr));

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 252, 36,
		"LOCATION", 8);
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 352, 36,
			current_machine->location, strlen(current_machine->location));

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 252, 24,
		"CONTACT", 7);
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 352, 24,
			current_machine->contact, strlen(current_machine->contact));

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 2, 60,
			"SUBNET", 6);
		gdk_draw_text(drawingarea1->window, main->font12b, gc, 2, 48,
			"GATEWAY", 7);
		sn = get_subnet();
		if (sn!=NULL)
		{
			sn->ip.sprint(buf2);
			sprintf(buf, "%s/%d", buf2, sn->mask);
			gdk_draw_text(drawingarea1->window, main->font12m, gc, 102, 60,
				buf, strlen(buf));
			gdk_draw_text(drawingarea1->window, main->font12m, gc, 102, 72,
				sn->name, strlen(sn->name));
			sn->gateway.sprint(buf);
			gdk_draw_text(drawingarea1->window, main->font12m, gc, 102, 48,
				buf, strlen(buf));
		}

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 252, 48,
			"LAST SEEN", 9);
		date_to_str(gmtime(&current_machine->last_seen), buf, 1);
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 352, 48,
			buf, strlen(buf));

		gdk_draw_text(drawingarea1->window, main->font12b, gc, 252, 60,
			"REGISTERED", 10);
		if (current_machine->registered == 0)
			strcpy(buf, "no"); else strcpy(buf, "yes");
		gdk_draw_text(drawingarea1->window, main->font12m, gc, 352, 60,
			buf, strlen(buf));

		row = 0;
		for (ln=current_machine->network->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			rtr = in->router;

			data[0] = strdup(rtr->name);

			sprint_mac(in->mac, buf);					
			data[1] = strdup(buf);

			sprintf(buf, "%s", in->port);
			data[2] = strdup(buf);

			gtk_clist_append((GtkCList*)clist20, data);
			delete data[0];	delete data[1];	delete data[2];

			gdkpixmap = gdk_pixmap_create_from_xpm_d(machine->window, &mask,
				&machine->style->bg[GTK_STATE_NORMAL], (gchar **)network_xpm);
			gtk_clist_set_pixtext(GTK_CLIST(clist20), row, 0, rtr->name, 4,
				gdkpixmap, mask);
			gdk_pixmap_unref(gdkpixmap);
			gdk_bitmap_unref(mask);
			row++;
		}
	}
}

Subnet *MachineWindow::get_subnet()
{
	Link *ln;
	Subnet *sn;

	for (ln=main->files->objects->subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (Subnet*) ln->item;
		if (in_subnet(current_machine->ip, sn->ip, sn->mask)) return sn;
	}
	return NULL;
}

void MachineWindow::machine_string(char *str)
{
/*
	gchar *text;
	int i;

	if (main->files->objects==NULL) return;

	for (i=0; i<main->files->objects->machines.size(); i++)
	{
		gtk_clist_get_text(GTK_CLIST(clist1), i, 0, &text);
		if (strcmp(text, str)==0)
		{
			gtk_clist_moveto(GTK_CLIST(clist1), i, 0, 0.5, 0);
			gtk_clist_select_row(GTK_CLIST(clist1), i, 0);
		}
	}
*/
}

