/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include "main.h"
#include "misc.h"
#include "subnet.h"
#include "router.h"
#include "network.h"
#include "machine.h"
#include "images/icon.xpm"

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((Search*)data)->hide();
}

static void on_Apply_activate(GtkWidget *w, gpointer data)
{
	gtk_entry_set_text(GTK_ENTRY(((Search*)data)->main->monitor->entry2), gtk_entry_get_text(GTK_ENTRY(((Search*)data)->entry2)));
	((Search*)data)->apply();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Search*)data)->hide();
	return (TRUE); 
}

static void title_cb(GtkWidget *clist, gint column, gpointer data)
{
	Search *snw = (Search*) data;
	snw->current_sort = column;

	gtk_clist_set_sort_column (GTK_CLIST (clist), column);
	gtk_clist_set_compare_func (GTK_CLIST (clist), NULL);
	gtk_clist_sort (GTK_CLIST (clist));
}

static gint expose_event(GtkWidget *w, GdkEventExpose *event, gpointer data)
{ ((Search*)data)->expose_search(); return FALSE; }

static void search_list_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Search *m = (Search*) data;
	Router *r;
	Machine *mc;
	gchar *text;

	m->ptr = gtk_clist_get_row_data(GTK_CLIST (clist), row);
	gtk_clist_get_text(GTK_CLIST(clist), row, 0, &text);
	m->type = 0;
	if (strcmp("Subnet", text)==0)
	{
		m->type = 1;
		m->main->subnet->current_subnet = (Subnet*) m->ptr;
		m->main->subnet->tags();
	}else
	if (strcmp("Device", text)==0)
	{
		m->type = 2;
		r = (Router*) m->ptr;
		r->selected = 1;
		m->main->monitor->draw();
	}else
	if (strcmp("Machine", text)==0)
	{
		m->type = 3;
		mc = (Machine*) m->ptr;
		mc->network->selected = 1;
		m->main->monitor->draw();
	}

	m->expose_search();
}

static void search_list_cb2(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Search *m = (Search*) data;
	m->ptr = NULL;
	m->type = 0;
	m->main->files->clear_tags();
	m->main->monitor->draw();
	m->expose_search();
}

Search::Search(Main *m)
{
	main = m;
	ptr = NULL;
	type = 0;
	current_sort = 0;
	strcpy(search_string, "");
	create();
	gc = gdk_gc_new(search->window);
}

Search::~Search()
{
}

void Search::create()
{
  search = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (search), "Network Search");
  gtk_widget_realize (search);
	window_icon(search, icon_xpm);
	gdk_window_set_icon_name(search->window, "Search");
	gtk_signal_connect(GTK_OBJECT(search), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  vbox12 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (search), vbox12);
  gtk_container_set_border_width (GTK_CONTAINER (vbox12), 4);

  hbox7 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox7, FALSE, TRUE, 0);

  label25 = gtk_label_new ("Enter Keyword");
  gtk_widget_show (label25);
  gtk_box_pack_start (GTK_BOX (hbox7), label25, FALSE, FALSE, 0);

  entry2 = gtk_entry_new ();
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (hbox7), entry2, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(entry2), "activate",
		GTK_SIGNAL_FUNC(on_Apply_activate), this);

  button8 = gtk_button_new_with_label ("Search");
  gtk_widget_show (button8);
  gtk_box_pack_start (GTK_BOX (hbox7), button8, FALSE, FALSE, 0);
  gtk_widget_set_usize (button8, 100, -2);

  scrolledwindow16 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow16);
  gtk_box_pack_start (GTK_BOX (vbox12), scrolledwindow16, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow16),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist15 = gtk_clist_new(2);
  gtk_widget_show (clist15);
  gtk_container_add (GTK_CONTAINER (scrolledwindow16), clist15);
  gtk_clist_set_column_width (GTK_CLIST (clist15), 0, 127);
  gtk_clist_set_column_width (GTK_CLIST (clist15), 1, 108);
  gtk_clist_column_titles_show (GTK_CLIST (clist15));
	gtk_signal_connect(GTK_OBJECT(clist15), "select_row",
		GTK_SIGNAL_FUNC(search_list_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist15), "unselect_row",
		GTK_SIGNAL_FUNC(search_list_cb2), this);
  gtk_signal_connect (GTK_OBJECT (clist15), "click_column",
		GTK_SIGNAL_FUNC (title_cb), this);

  label26 = gtk_label_new ("Record");
  gtk_widget_show (label26);
  gtk_clist_set_column_widget (GTK_CLIST (clist15), 0, label26);
  gtk_misc_set_alignment (GTK_MISC (label26), 0, 0.5);

  label27 = gtk_label_new ("Name");
  gtk_widget_show (label27);
  gtk_clist_set_column_widget (GTK_CLIST (clist15), 1, label27);
  gtk_misc_set_alignment (GTK_MISC (label27), 0, 0.5);

	drawingarea10 = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea10);
  gtk_box_pack_start (GTK_BOX (vbox12), drawingarea10, FALSE, TRUE, 0);
  gtk_widget_set_usize (drawingarea10, 500, 88);
	gtk_signal_connect (GTK_OBJECT (drawingarea10), "expose_event",
		(GtkSignalFunc) expose_event, this);

  hbox12 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox12);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox12, FALSE, TRUE, 0);

  button15 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button15);
  gtk_box_pack_end (GTK_BOX (hbox12), button15, FALSE, FALSE, 0);
  gtk_widget_set_usize (button15, 100, -2);

  button16 = gtk_button_new_with_label ("Print");
  gtk_widget_show (button16);
  gtk_box_pack_end (GTK_BOX (hbox12), button16, FALSE, FALSE, 0);
  gtk_widget_set_usize (button16, 100, -2);

  gtk_signal_connect (GTK_OBJECT (button8), "clicked",
		GTK_SIGNAL_FUNC (on_Apply_activate), this);
  gtk_signal_connect (GTK_OBJECT (button15), "clicked",
		GTK_SIGNAL_FUNC (on_Close_activate), this);
}

void Search::show()
{
	type = 0;
	ptr = NULL;
	gtk_widget_show(search);
	gdk_window_raise(search->window);
}

void Search::hide()
{
	gtk_widget_hide(search);
}

void Search::apply()
{
	Link *ln3;
	Subnet *sn;
	Router *r;
	Machine *m;
	gchar *data[2];
	char buf[256];
	char buf2[256];
	int row;

	if (main->files->objects==NULL) return;

	gchar *txt = gtk_entry_get_text(GTK_ENTRY(entry2));
	strcpy(search_string, txt);
	ptr = NULL;
	type = 0;
	expose_search();

	row=0;
	gtk_clist_clear(GTK_CLIST(clist15));
	for (ln3=main->files->objects->subnets.get_head(); ln3!=NULL; ln3=ln3->next)
	{
		sn = (Subnet*) ln3->item;
		sn->match(txt);
		if (sn->find > 0)
		{
			sn->ip.sprint(buf); sprintf(buf2, "%s/%d", buf, sn->mask);
			data[0] = strdup("Subnet");
			if (strcmp(sn->name, "")!=0)
				data[1] = strdup(sn->name);
			else
				data[1] = strdup(buf);
			gtk_clist_append(GTK_CLIST(clist15), data);
			delete data[0]; delete data[1];
			gtk_clist_set_row_data(GTK_CLIST(clist15), row, sn);
			row++;
		}
	}
	for (ln3=main->files->objects->routers.get_head(); ln3!=NULL; ln3=ln3->next)
	{
		r = (Router*) ln3->item;
		r->match(txt);
		if (r->find > 0)
		{
			data[0] = strdup("Device");
			data[1] = strdup(r->name);
			gtk_clist_append(GTK_CLIST(clist15), data);
			delete data[0]; delete data[1];
			gtk_clist_set_row_data(GTK_CLIST(clist15), row, r);
			row++;
		}
	}

	for (ln3=main->files->objects->machines.get_head(); ln3!=NULL; ln3=ln3->next)
	{
		m = (Machine*) ln3->item;
		m->match(txt);
		if (m->find > 0)
		{
			data[0] = strdup("Machine");
			sprint_mac(m->mac, buf);
			m->ip.sprint(buf2);
			if (strcmp(m->name, "")!=0)
			{
				data[1] = strdup(m->name);
			}else
			if ((strcmp(buf2, "")!=0) && (strcmp(buf2, "0.0.0.0")!=0))
			{
				data[1] = strdup(buf2);
			}else
				data[1] = strdup(buf);
			gtk_clist_append(GTK_CLIST(clist15), data);
			delete data[0]; delete data[1];
			gtk_clist_set_row_data(GTK_CLIST(clist15), row, m);
			row++;
		}
	}
}

void Search::in_match(char *txt, char *lbl, int y)
{
	int i=0;

	if (lower_contains(txt, search_string))
	{
		gdk_gc_set_foreground(gc, &main->red);
		i=1;
	}
	gdk_draw_text(drawingarea10->window, main->font12m, gc, 152, y,
		txt, strlen(txt));
	if (i==1)
		gdk_gc_set_foreground(gc, &main->black);
	gdk_draw_text(drawingarea10->window, main->font12b, gc, 2, y,
			lbl, strlen(lbl));
}

void Search::expose_search()
{
	Machine *m;
	Router *r;
	Subnet *s;
	char buf[256], buf2[256];
	int x, y, w, h;
	GtkStyle *style;

	x = drawingarea10->allocation.width;
	y = drawingarea10->allocation.height;
	w = x/128;
	h = y/128;
	style = gtk_widget_get_style(search);
	for (x=0; x<=w; x++)
		for (y=0; y<=h; y++)
			gtk_draw_flat_box(style, drawingarea10->window, GTK_STATE_NORMAL,
				GTK_SHADOW_NONE, x*128, y*128, 128, 128);

	if (type==1)
	{
		s = (Subnet*) ptr;
		s->ip.sprint(buf);
		sprintf(buf2, "%s/%d", buf, s->mask);
		gdk_gc_set_foreground(gc, &main->black);
		gdk_draw_text(drawingarea10->window, main->font12b, gc, 2, 12,
			"Subnet", 6);
		in_match(s->name, "Description", 24);
		in_match(buf2, "IP Address / Mask", 36);
	}else
	if (type==2)
	{
		r = (Router*) ptr;
		gdk_gc_set_foreground(gc, &main->black);
		gdk_draw_text(drawingarea10->window, main->font12b, gc, 2, 12,
			"Device", 6);
		in_match(r->name, "Name", 24);
		in_match(r->descr, "Description", 36);
		in_match(r->location, "Location", 48);
		in_match(r->contact, "Contact", 60);
	}else
	if (type==3)
	{
		m = (Machine*) ptr;
		m->ip.sprint(buf);
		sprint_mac(m->mac, buf2);
		gdk_gc_set_foreground(gc, &main->black);
		gdk_draw_text(drawingarea10->window, main->font12b, gc, 2, 12,
			"Machine", 7);
		in_match(m->name, "Name", 24);
		in_match(buf2, "MAC Address", 36);
		in_match(buf, "IP Address", 48);
		in_match(m->descr, "Description", 60);
		in_match(m->location, "Location", 72);
		in_match(m->contact, "Contact", 84);
	}
}

