/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <string.h>
#include <stdlib.h>
#include "data.h"
#include "router.h"
#include "network.h"
#include "machine.h"
#include "interface.h"
#include "subnet.h"
#include "misc.h"

Data::Data()
{
}

Data::~Data()
{
	Network *n;
	Machine *m;
	Subnet *s;
	Router *r;
	Link *ln;

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{ s = (Subnet*) ln->item; delete s; }
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{ m = (Machine*) ln->item; delete m; }
	for (ln=routers.get_head(); ln!=NULL; ln=ln->next)
	{ r = (Router*) ln->item; delete r; }
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{ n = (Network*) ln->item; delete n; }
}

void Data::save(FILE *f)
{
	int k;
	Network *n;
	Subnet *s;
	Router *r;
	Link *ln;

	write_int(f, subnets.size());
	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (Subnet*) ln->item;
		s->save(f);
	}

	write_int(f, routers.size());
	for (ln=routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		r->save(f);
	}

	// save only those networks that are attached to stuff
	k = 0;
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() + n->machines.size() > 1) k++;
	}
	write_int(f, k);
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() + n->machines.size() > 1)
			n->save(f);
	}
}

void Data::load(FILE *f)
{
	int i, j;
	Network *n;
	Router *r;
	Subnet *s;

	j = read_int(f);
	subnets.clear();
	for (i=0; i<j; i++)
	{
		s = new Subnet();
		s->load(f);
		subnets.tag(s);
	}

	j = read_int(f);
	routers.clear();
	for (i=0; i<j; i++)
	{
		r = new Router();
		r->load(f);
		routers.tag(r);
	}

	j = read_int(f);
	networks.clear();
	for (i=0; i<j; i++)
	{
		n = new Network();
		n->load(f, &routers, &machines, &subnets);
		networks.tag(n);
	}
}

void Data::print()
{
	printf("SUBNETS %d\n", subnets.size());
	printf("ROUTERS %d\n", routers.size());
	printf("MACHINES %d\n", machines.size());
	printf("NETWORKS %d\n", networks.size());
}

void Data::savexml(char *fname)
{
	xmlDocPtr doc;
	Subnet *s;
	Router *r;
	Network *n;
	Link *ln;

	doc = xmlNewDoc((xmlChar*)"1.0");
	doc->root = xmlNewDocNode(doc, NULL, (xmlChar*)"Nomad_Topology", NULL);

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (Subnet*) ln->item;
		s->savexml(doc->root);
	}
	for (ln=routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		r->savexml(doc->root);
	}

	// save only those networks that are attached to stuff
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() + n->machines.size() > 1)
			n->savexml(doc->root);
	}

	xmlSaveFile(fname, doc);
	xmlFreeDoc(doc);
}

void Data::loadxml(char *fname)
{
	xmlDocPtr doc;
	xmlNodePtr xmlnode;
	Router *r;
	Subnet *s;
	Network *n;

	doc = xmlParseFile(fname);

	if (!doc || !doc->root || !doc->root->name ||
			strcmp((char*)doc->root->name, "Nomad_Topology")!=0)
	{
		xmlFreeDoc(doc);
		return;
	}

	xmlnode = doc->root->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Subnet")==0)
			{
				s = new Subnet();
				s->loadxml(xmlnode);
				subnets.tag(s);
			}else
			if (strcmp((char*)xmlnode->name, "Device")==0)
			{
				r = new Router();
				r->loadxml(xmlnode);
				routers.tag(r);
			}else
			if (strcmp((char*)xmlnode->name, "Network")==0)
			{
				n = new Network();
				n->loadxml(xmlnode, &routers, &machines, &subnets);
				networks.tag(n);
			}
		}
		xmlnode = xmlnode->next;
	}

	xmlFreeDoc(doc);
}

