/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "ipaddress.h"
#include "interface.h"
#include "network.h"
#include "misc.h"

Interface::Interface()
{
	clear();
}

Interface::~Interface()
{
}

void Interface::clear()
{
	int i;

	index = 0;
	for (i=0; i<6; i++) mac[i] = 0;
	speed = 0;
	type = 0;
	status = 0;
	strcpy(descr, "");
	strcpy(port, "");
	network = NULL;
	router = NULL;
}

void Interface::print()
{
	printf("  Index %d, mac ", index);
	print_mac(mac);
	printf(", speed %lu, type %d, status %d, descr \"%s\"\n",
		speed, type, status, descr);
}

void Interface::save(FILE *f)
{
	int i;
	IPAddress *tmpip;
	Link *ln;

	write_int(f, index);
	for (i=0; i<6; i++)  fputc(mac[i], f);
	write_ulong(f, speed);
	write_int(f, type);
	write_int(f, status);
	for (i=0; i<64; i++)  fputc(descr[i], f);
	for (i=0; i<64; i++)  fputc(port[i], f);
	write_int(f, ip_addresses.size());
	for (ln=ip_addresses.get_head(); ln!=NULL; ln=ln->next)
	{
		tmpip = (IPAddress*) ln->item;
		tmpip->save(f);
	}
}

void Interface::load(FILE *f)
{
	int i, j;
	IPAddress *tmpip;

	index = read_int(f);
	for (i=0; i<6; i++)  mac[i] = fgetc(f);
	speed = read_ulong(f);
	type = read_int(f);
	status = read_int(f);
	for (i=0; i<64; i++) descr[i] = fgetc(f); descr[64] = '\0';
	for (i=0; i<64; i++) port[i] = fgetc(f); port[64] = '\0';
	j = read_int(f);
	for (i=0; i<j; i++)
	{
		tmpip = new IPAddress();
		tmpip->load(f);
		ip_addresses.tag(tmpip);
	}
}

void Interface::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	IPAddress *tmpip;
	char buf[256];
	Link *ln;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Interface", NULL);
	sprintf(buf, "%d", index);
	xmlSetProp(newxml, (xmlChar*)"index", (xmlChar*)buf);
	sprint_mac(mac, buf);
	xmlSetProp(newxml, (xmlChar*)"mac", (xmlChar*)buf);
	sprintf(buf, "%lu", speed);
	xmlSetProp(newxml, (xmlChar*)"speed", (xmlChar*)buf);
	sprintf(buf, "%d", type);
	xmlSetProp(newxml, (xmlChar*)"type", (xmlChar*)buf);
	sprintf(buf, "%d", status);
	xmlSetProp(newxml, (xmlChar*)"status", (xmlChar*)buf);
	xmlSetProp(newxml, (xmlChar*)"description", (xmlChar*)descr);
	xmlSetProp(newxml, (xmlChar*)"port", (xmlChar*)port);
	for (ln=ip_addresses.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Address", NULL);
		tmpip = (IPAddress*) ln->item;	tmpip->sprint(buf);
		xmlSetProp(cur, (xmlChar*)"ip", (xmlChar*)buf);
	}
}

void Interface::loadxml(xmlNodePtr xmlnode)
{
	IPAddress *tmpip;
	xmlChar *buf;

	// check this is the correct  node
	if (!xmlnode->name || strcmp((char*)xmlnode->name, "Interface")!=0)
		return;

	// read properties
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"index");
	sscanf((char*)buf, "%d", &index);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mac");
	convert_mac_3((char*)buf, mac);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"speed");
	sscanf((char*)buf, "%lu", &speed);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"type");
	sscanf((char*)buf, "%d", &type);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"status");
	sscanf((char*)buf, "%d", &status);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"description");
	strcpy(descr, (char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"port");
	strcpy(port, (char*)buf);	free(buf);

	ip_addresses.clear();
	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Address")==0)
			{
				tmpip = new IPAddress();
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip");
				tmpip->set((char*)buf);	free(buf);
				ip_addresses.tag(tmpip);
			}
		}
		xmlnode = xmlnode->next;
	}
}

