/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include "ipaddress.h"
#include "misc.h"

IPAddress::IPAddress()
{
	clear();
}

IPAddress::~IPAddress()
{
}

void IPAddress::clear()
{
	int i;
	for (i=0; i<4; i++) ip[i] = 0;
}

void IPAddress::save(FILE *f)
{
	int i;
	for (i=0; i<4; i++)  fputc(ip[i], f);
}

void IPAddress::load(FILE *f)
{
	int i;
	for (i=0; i<4; i++)  ip[i] = fgetc(f);
}

void IPAddress::print()
{
	printf("%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
}

void IPAddress::sprint(char *buf)
{
	sprintf(buf, "%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
}

void IPAddress::set(char *str)
{
	int a, b, c, d;
	if (sscanf(str, "%d.%d.%d.%d", &a, &b, &c, &d)!=4) return;
	ip[0] = a;
	ip[1] = b;
	ip[2] = c;
	ip[3] = d;
}

void IPAddress::set(IPAddress pi)
{
	ip[0] = pi.ip[0];
	ip[1] = pi.ip[1];
	ip[2] = pi.ip[2];
	ip[3] = pi.ip[3];
}

int IPAddress::zero()
{
	if ((ip[0] == 0) && (ip[1] == 0) && (ip[2] == 0) && (ip[3] == 0))
		return 1;
	return 0;
}

int IPAddress::compare(IPAddress pi)
{
	if ((ip[0] == pi.ip[0]) && (ip[1] == pi.ip[1]) &&
			(ip[2] == pi.ip[2]) && (ip[3] == pi.ip[3]))
		return 1;
	return 0;
}

int IPAddress::real_compare(IPAddress pi)
{
	if (ip[0] != pi.ip[0]) return (ip[0]-pi.ip[0]);
	if (ip[1] != pi.ip[1]) return (ip[1]-pi.ip[1]);
	if (ip[2] != pi.ip[2]) return (ip[2]-pi.ip[2]);
	if (ip[3] != pi.ip[3]) return (ip[3]-pi.ip[3]);
	return 0;
}

int IPAddress::compare(char *str)
{
	IPAddress pi;
	pi.set(str);
	if ((ip[0] == pi.ip[0]) && (ip[1] == pi.ip[1]) &&
			(ip[2] == pi.ip[2]) && (ip[3] == pi.ip[3]))
		return 1;
	return 0;
}

int IPAddress::in_range(char *str1, char *str2)
{
	IPAddress s1, s2;
	unsigned long l1, l2, l3;

	s1.set(str1);
	s2.set(str2);

	l1 = (((((s1.ip[0]*256)+s1.ip[1])*256)+s1.ip[2])*256)+s1.ip[3];
	l2 = (((((s2.ip[0]*256)+s2.ip[1])*256)+s2.ip[2])*256)+s2.ip[3];
	l3 = (((((ip[0]*256)+ip[1])*256)+ip[2])*256)+ip[3];

	if ((l1<=l3) && (l2>=l3))
		return 1;

	return 0;
}

