/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include "misc.h"

time_t make_time_from_string(char *buf)
{
	struct tm ptm;
	char year[8], month[8], day[8], hour[8], minute[8], second[8];
	int i;

	if ((buf==NULL) || (strlen(buf)!=14)) return 0;

// printf("TIME %s\n", buf);

	for (i=0; i<4; i++) year[i] = buf[i];      year[4] = '\0';
	for (i=0; i<2; i++) month[i] = buf[i+4];   month[2] = '\0';
	for (i=0; i<2; i++) day[i] = buf[i+6];     day[2] = '\0';
	for (i=0; i<2; i++) hour[i] = buf[i+8];    hour[2] = '\0';
	for (i=0; i<2; i++) minute[i] = buf[i+10]; minute[2] = '\0';
	for (i=0; i<2; i++) second[i] = buf[i+12]; second[2] = '\0';

	sscanf(year, "%d", &i);			ptm.tm_year = i-1900;
	sscanf(month, "%d", &i);		ptm.tm_mon = i-1;
	sscanf(day, "%d", &i);			ptm.tm_mday = i;
	sscanf(hour, "%d", &i);			ptm.tm_hour = i;
	sscanf(minute, "%d", &i);		ptm.tm_min = i;
	sscanf(second, "%d", &i);		ptm.tm_sec = i;
															ptm.tm_isdst = 0;

	return mktime(&ptm);
}

void make_string_from_time(time_t t, char *buf)
{
	struct tm *gmt;
	int d, m, y, h, n, s;

	gmt = gmtime(&t);
	y = gmt->tm_year+1900;
	m = gmt->tm_mon+1;
	d = gmt->tm_mday;
	h = gmt->tm_hour;
	n = gmt->tm_min;
	s = gmt->tm_sec;
	sprintf(buf, "%04d%02d%02d%02d%02d%02d", y, m, d, h, n, s);
}

int starts_with(char *str, char *bigstr)
{
	char buf[256];
	strcpy(buf, bigstr);
	buf[strlen(str)] = '\0';
	return (strcmp(buf, str)==0);
}

void write_int(FILE *f, int val)
{
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

void write_ulong(FILE *f, unsigned long val)
{
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

void write_time_t(FILE *f, time_t val)
{
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

int read_int(FILE *f)
{
	int a, b, c, d;
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

unsigned long read_ulong(FILE *f)
{
	int a, b, c, d;
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

time_t read_time_t(FILE *f)
{
	int a, b, c, d;
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

void convert_mac(char *macin, unsigned char *macout)
{
	int aa, bb, cc, dd, ee, ff, i;

	for (i=0; i<6; i++) macout[i] = '\0';
	if (sscanf(macin, "\"%x %x %x %x %x %x \"",
		&aa, &bb, &cc, &dd, &ee, &ff)!=6) return;

//	printf("#%x# #%x# #%x# #%x# #%x# #%x#\n", aa, bb, cc, dd, ee, ff);

	macout[0] = aa; macout[1] = bb; macout[2] = cc;
	macout[3] = dd; macout[4] = ee; macout[5] = ff;

	if ((aa==255) && (bb==255) && (cc==255) &&
			(dd==255) && (ee==255) && (ff==255))
		for (i=0; i<6; i++) macout[i] = '\0';
}

void convert_mac_2(char *macin, unsigned char *macout)
{
	int aa, bb, cc, dd, ee, ff, i;

	for (i=0; i<6; i++) macout[i] = '\0';
	if (sscanf(macin, "%d.%d.%d.%d.%d.%d",
		&aa, &bb, &cc, &dd, &ee, &ff)!=6) return;

	macout[0] = aa; macout[1] = bb; macout[2] = cc;
	macout[3] = dd; macout[4] = ee; macout[5] = ff;

	if ((aa==255) && (bb==255) && (cc==255) &&
			(dd==255) && (ee==255) && (ff==255))
		for (i=0; i<6; i++) macout[i] = '\0';
}

void convert_mac_3(char *macin, unsigned char *macout)
{
	int i, v[6];
	char buf[4];

	for (i=0; i<6; i++) macout[i] = '\0';
	if (strlen(macin)!=12) return;
	for (i=0; i<6; i++)
	{
		buf[0] = macin[i*2];
		buf[1] = macin[(i*2)+1];
		buf[2] = '\0';
		if (sscanf(buf, "%x", &v[i])!=1) return;
	}
	for (i=0; i<6; i++) macout[i] = v[i];
}

int compare_mac(unsigned char *mac1, unsigned char *mac2)
{
	if ((mac1[0] == mac2[0]) && (mac1[1] == mac2[1]) &&
			(mac1[2] == mac2[2]) && (mac1[3] == mac2[3]) &&
			(mac1[4] == mac2[4]) && (mac1[5] == mac2[5]))
		return 1;
	return 0;
}

int zero_mac(unsigned char *mac)
{
	return ((mac[0] == 0) && (mac[1] == 0) && (mac[2] == 0) &&
					(mac[3] == 0) && (mac[4] == 0) && (mac[5] == 0));
}

int mask_num(char *buf)
{
	int a, b, c, d, count = 0;
	unsigned long total;
	if (sscanf(buf, "%d.%d.%d.%d", &a, &b, &c, &d) != 4) return -1;
	total = (((((a*256)+b)*256)+c)*256)+d;
	if (total==0) return 32;
	while (total%2 == 0) { total /= 2; count++; }
	return 32-count;
}

void print_mac(unsigned char *buf)
{
	printf("%02x%02x%02x%02x%02x%02x", buf[0], buf[1], buf[2], buf[3], buf[4], buf[5]);
}

void sprint_mac(unsigned char *buf, char *str)
{
	char tmp[16];
	sprintf(tmp, "%02x%02x%02x%02x%02x%02x", buf[0], buf[1], buf[2], buf[3], buf[4], buf[5]);
	strcpy(str, tmp);
}

void print_time(time_t buf)
{
	struct tm *t;
	t = gmtime(&buf);
	printf("%02d/%02d/%04d %02d:%02d:%02d", t->tm_mday, t->tm_mon+1, t->tm_year+1900, t->tm_hour, t->tm_min, t->tm_sec);
}

int mask_power(int pwr)
{
	int i, current = 1;

	if (pwr<0) return -1;
	if (pwr==0) return 1;
	for (i=0; i<pwr; i++) current *= 2;
	return current;
}

int in_subnet(IPAddress ip, IPAddress sn, int mask)
{
	unsigned long lip, lsn;
	int mv;

	lip = (((((ip.ip[0]*256)+ip.ip[1])*256)+ip.ip[2])*256)+ip.ip[3];
	lsn = (((((sn.ip[0]*256)+sn.ip[1])*256)+sn.ip[2])*256)+sn.ip[3];
	mv = mask_power(32-mask);

// printf("Subnet stuff ip %ul, subnet %ul, mask %d\n", lip, lsn, mv);

	if ((lip>lsn) && (lip<lsn+mv))
		return 1;

	return 0;
}

int contains(char *str, char *substr)
{
	int a, b, i, j;
	char buf[256];

	a = strlen(str);
	b = strlen(substr);

	for (i=0; i<a-b+1; i++)
	{
		for (j=0; j<b; j++)
		{
			buf[j] = str[j+i];
		}
		buf[b] = '\0';
		if ((strcmp(buf, substr))==0) return 1;
	}

	return 0;
}

int lower_contains(char *str, char *str2)
{
	int a, b, i;
	char buf1[256];
	char buf2[256];

	strcpy(buf1, str);
	strcpy(buf2, str2);

	a = strlen(buf1);
	b = strlen(buf1);

	for (i=0; i<a; i++)
		if ((buf1[i]>='A') && (buf1[i]<='Z')) buf1[i] = buf1[i] - 'A' + 'a';
	for (i=0; i<b; i++)
		if ((buf2[i]>='A') && (buf2[i]<='Z')) buf2[i] = buf2[i] - 'A' + 'a';
	buf1[a] = '\0'; buf2[b] = '\0';

	return contains(buf1, buf2);
}

void date_to_str(struct tm *tb, char *dout, int type)
{
	char day[16];
	char ext[8];
	char month[16];

	if (tb->tm_wday == 0) strcpy(day, "Sun"); else
	if (tb->tm_wday == 1) strcpy(day, "Mon"); else
	if (tb->tm_wday == 2) strcpy(day, "Tue"); else
	if (tb->tm_wday == 3) strcpy(day, "Wed"); else
	if (tb->tm_wday == 4) strcpy(day, "Thu"); else
	if (tb->tm_wday == 5) strcpy(day, "Fri"); else
		strcpy(day, "Sat");

	if ((tb->tm_mday == 1) || (tb->tm_mday == 21) || (tb->tm_mday ==31))
		strcpy(ext, "st"); else
	if ((tb->tm_mday == 2) || (tb->tm_mday == 22))
		strcpy(ext, "nd"); else
	if ((tb->tm_mday == 3) || (tb->tm_mday == 23))
		strcpy(ext, "rd"); else strcpy(ext, "th");

	if (tb->tm_mon == 0) strcpy(month, "Jan"); else
	if (tb->tm_mon == 1) strcpy(month, "Feb"); else
	if (tb->tm_mon == 2) strcpy(month, "Mar"); else
	if (tb->tm_mon == 3) strcpy(month, "Apr"); else
	if (tb->tm_mon == 4) strcpy(month, "May"); else
	if (tb->tm_mon == 5) strcpy(month, "Jun"); else
	if (tb->tm_mon == 6) strcpy(month, "Jul"); else
	if (tb->tm_mon == 7) strcpy(month, "Aug"); else
	if (tb->tm_mon == 8) strcpy(month, "Sep"); else
	if (tb->tm_mon == 9) strcpy(month, "Oct"); else
	if (tb->tm_mon == 10) strcpy(month, "Nov"); else
	if (tb->tm_mon == 11) strcpy(month, "Dec");

	if (type==1)
		sprintf(dout, "%s %d%s %s %d %d:%02d:%02d", day, tb->tm_mday, ext, month, tb->tm_year+1900, tb->tm_hour, tb->tm_min, tb->tm_sec);
	else
	if (type==2)
		sprintf(dout, "%s %d%s %s %d", day, tb->tm_mday, ext, month, tb->tm_year+1900);
	else
	if (type==3)
		sprintf(dout, "%s %d%s %s", day, tb->tm_mday, ext, month);
	else
	if (type==4)
		sprintf(dout, "%s %d%s", day, tb->tm_mday, ext);
	else
	if (type==5)
		sprintf(dout, "%s", day);
	else
	if (type==6)
		sprintf(dout, "%c", day[0]);
}

int compare_ip_addresses(char *r1, char *r2)
{
	unsigned long total1;
	unsigned long total2;
	int a, b, c, d;
	if (sscanf(r1, "%d.%d.%d.%d", &a, &b, &c, &d) != 4) return 0;
	total1 = (((((a*256)+b)*256)+c)*256)+d;
	if (sscanf(r2, "%d.%d.%d.%d", &a, &b, &c, &d) != 4) return 0;
	total2 = (((((a*256)+b)*256)+c)*256)+d;

	if (total1>total2) return 1; else
	if (total1<total2) return -1; else
	return 0;
}

void speed_to_str(unsigned long spd, char *buf)
{
	if (spd == 0)
		sprintf(buf, "0");
	else
	if ((spd%1000000000) == 0)
		sprintf(buf, "%luGb", (unsigned long) (spd/1000000000));
	else
	if ((spd%1000000) == 0)
		sprintf(buf, "%luMb", (unsigned long) (spd/1000000));
	else
	if ((spd%1000) == 0)
		sprintf(buf, "%lukb", (unsigned long) (spd/1000));
	else
		sprintf(buf, "%lu", spd);
}

void interface_type_to_str(int type, char *buf)
{
	char tmp[256];

	switch (type)
	{
		case 1:		sprintf(tmp, "Other"); break;
		case 6: 	sprintf(tmp, "Ethernet"); break;
		case 15:	sprintf(tmp, "FDDI"); break;
		case 18:	sprintf(tmp, "E1"); break;
		case 22:	sprintf(tmp, "Serial"); break;
		case 23:	sprintf(tmp, "PPP Serial"); break;
		case 28:	sprintf(tmp, "SLIP"); break;
		case 37:	sprintf(tmp, "ATM Layer"); break;
		case 39:	sprintf(tmp, "SONET / SDH"); break;
		case 49:	sprintf(tmp, "AAL5 Layer"); break;
//		case 53:	sprintf(tmp, ""); break;
		case 59:	sprintf(tmp, "LEC"); break;
		case 24:	sprintf(tmp, "Loopback"); break;
		default:	sprintf(tmp, "Unknown");
	}
	sprintf(buf, "%s (%d)", tmp, type);
}

int lock(char *fname)
{
	int st, lock = -1;

	lock=open(fname, O_RDWR | O_CREAT, 00664);
	while (lock == -1)
	{
		sleep(1);
		lock=open(fname, O_RDWR | O_CREAT, 00664);
	}

	lseek(lock, 0L, 0);
	st = lockf(lock, F_LOCK, 0L);

	return lock;
}

void unlock(int fd)
{
	int st;
	int size1 = 0, size2 = 0, val;
	FILE *f1, *f2;

	st = lockf(fd, F_ULOCK, 0);
	while (st == -1)
	{
		sleep(1);
		st = lockf(fd, F_ULOCK, 0);
	}

	close(fd);
}

