/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>
#include "machine.h"
#include "interface.h"
#include "router.h"
#include "network.h"
#include "main.h"
#include "misc.h"

char* usage = "<option>\n\n\
    a - Generate list of unregistered machines\n\
    b - Generate list of missing machines\n\
    c - Generate list of missing MAC addresses\n\
    d - Generate list of discovered machines\n\
";

int main(int argc, char *argv[])
{
	Main m;

	if (argc < 2)
	{
		printf("usage: %s %s\n", argv[0], usage);
		return 0;
	}

	if (m.load()==0)
	{
		printf("You need to have a network file\n");
		return 0;
	}

	switch (argv[1][0])
	{
		case 'a': m.unreg(); break;
		case 'b': m.missing(); break;
		case 'c': m.mac(); break;
		case 'd': m.all(); break;
		default : printf("usage: %s %s\n", argv[0], usage);
	}

	return 1;
}

int compare_machines(Link *a, Link *b)
{
	Machine *m1, *m2;

	m1 = (Machine*) a->item;
	m2 = (Machine*) b->item;

	int val = m1->ip.real_compare(m2->ip);

	if (val==0)
		return strcmp(m1->name, m2->name);
	else
		return val;
}

int compare_unreg(Link *a, Link *b)
{
	Machine *m1, *m2;
	Interface *in;
	char buf[256];
	char buf2[256];
	Link *ln;

	m1 = (Machine*) a->item;
	m2 = (Machine*) b->item;

	int val = m1->ip.real_compare(m2->ip);

	if (val==0)
	{
		ln = m1->network->interfaces.get_head();
		in = (Interface*) ln->item;
		sprintf(buf, "%s %s", in->router->name, in->port);
		ln = m2->network->interfaces.get_head();
		in = (Interface*) ln->item;
		sprintf(buf2, "%s %s", in->router->name, in->port);
		return strcmp(buf, buf2);
	}else
		return val;
}

int Main::load()
{
	struct passwd *p;
	char buf[256];
//	FILE *f;

	p = getpwuid(getuid());
	sprintf(buf, "%s/%s/network", p->pw_dir, HOME_DIR);
/*
	f = fopen(buf, "r");
	if (!f) return 0;
	newobj.load(f);
	fclose(f);
*/
	newobj.loadxml(buf);
	return 1;
}

void Main::unreg()
{
	char buf2[256];
	char buf3[256];
	int ips;
	Link *ln, *ln2;
	Machine *m;
	Interface *in;
	List lst;

	ips = 0;
	lst.clear();
	for (ln=newobj.machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m->registered == 0)
		{
			if (m->ip.zero()==0) ips++;
			lst.tag(m);
		}
	}

	lst.sort(compare_unreg, BUBBLE_SORT);

	printf("There are %d machines detected, %d of them are unregistered\n",
		newobj.machines.size(), lst.size());
	printf("and of the unregistered machines %d are using IP addresses\n", ips);
	for (ln=lst.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->ip.sprint(buf2);
		sprint_mac(m->mac, buf3);
		ln2 = m->network->interfaces.get_head();
		in = (Interface*) ln2->item;
		printf("%12s %-15s %s %s\n", buf3, buf2, in->router->name, in->port);
	}
}

void Main::missing()
{
	struct passwd *p;
	char buf[256];
	char mname[256];
	char mmac[256];
	char mip[256];
	char mcontact[256];
	char mlocation[256];
	char mdescr[256];
	FILE *f;
	int i, count, total;
	Link *ln;
	Machine *m;
	unsigned char tmpmac[6];
	IPAddress tmpip;
	Machine *t;
	List lst;

	sprintf(buf, "%s/%s/machines", p->pw_dir, HOME_DIR);
	f = fopen(buf, "r");
	if (!f)
	{
		printf("You need to have a machines file\n");
		return;
	}

	total = 0;
	while(fscanf(f, "%[^,],%[^,],%[^,],%[^,],%[^,],%[^\n]\n",
			mname, mmac, mip, mcontact, mlocation, mdescr)==6)
	{
		convert_mac_3(mmac, tmpmac);
		count = 0;
		for (ln=newobj.machines.get_head(); ln!=NULL; ln=ln->next)
		{
			m = (Machine*) ln->item;
			if (compare_mac(m->mac, tmpmac)==1)
			{
				count++;
				break;
			}
		}
		if (count==0)
		{
			t = new Machine();
			strcpy(t->name, mname);
			for (i=0; i<6; i++) t->mac[i] = tmpmac[i];
			t->ip.set(mip);
			strcpy(t->contact, mcontact);
			strcpy(t->location, mlocation);
			strcpy(t->descr, mdescr);
			lst.tag(t);
		}
		total++;
	}
	fclose(f);

	lst.sort(compare_machines, BUBBLE_SORT);

	printf("There are %d machines listed, %d are missing\n",
		total, lst.size());
	for (ln=lst.get_head(); ln!=NULL; ln=ln->next)
	{
		t = (Machine*) ln->item;
		t->ip.sprint(mip);
		sprint_mac(t->mac, mmac);
		printf("%-16s %12s %-15s %s\n", t->name, mmac, mip, t->location);
	}
}

void Main::mac()
{
	struct passwd *p;
	char buf[256];
	char mname[256];
	char mmac[256];
	char mip[256];
	char mcontact[256];
	char mlocation[256];
	char mdescr[256];
	FILE *f;
	int i, count, total;
	Link *ln;
	Machine *m;
	IPAddress tmpip;
	Machine *t;
	List lst;

	sprintf(buf, "%s/%s/machines", p->pw_dir, HOME_DIR);
	f = fopen(buf, "r");
	if (!f)
	{
		printf("You need to have a machines file\n");
		return;
	}

	total = 0;
	while(fscanf(f, "%[^,],%[^,],%[^,],%[^,],%[^,],%[^\n]\n",
			mname, mmac, mip, mcontact, mlocation, mdescr)==6)
	{
		if ((strcmp("000000000000", mmac)==0) && (strcmp("0.0.0.0", mip)!=0))
		{
			tmpip.set(mip);
			count = 0;
			for (ln=newobj.machines.get_head(); ln!=NULL; ln=ln->next)
			{
				m = (Machine*) ln->item;
				if (tmpip.compare(m->ip)==1)
				{
					if (!zero_mac(m->mac))
					{
						count++;
						break;
					}
				}
			}
			if (count>0)
			{
				t = new Machine();
				strcpy(t->name, mname);
				for (i=0; i<6; i++) t->mac[i] = m->mac[i];
				t->ip.set(mip);
				strcpy(t->contact, mcontact);
				strcpy(t->location, mlocation);
				strcpy(t->descr, mdescr);
				lst.tag(t);
			}
		}
		total++;
	}
	fclose(f);

	lst.sort(compare_machines, BUBBLE_SORT);

	printf("There are %d machines listed, discovered %d missing MAC addresses\n",
		total, lst.size());
	for (ln=lst.get_head(); ln!=NULL; ln=ln->next)
	{
		t = (Machine*) ln->item;
		t->ip.sprint(mip);
		sprint_mac(t->mac, mmac);
		printf("%-16s %12s %-15s %s\n", t->name, mmac, mip, t->location);
	}
}

void Main::all()
{
	char buf2[256];
	char buf3[256];
	Link *ln;
	Machine *m;

	newobj.machines.sort(compare_machines, BUBBLE_SORT);

	printf("There are %d discovered machines\n", newobj.machines.size());
	for (ln=newobj.machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->ip.sprint(buf2);
		sprint_mac(m->mac, buf3);
		printf("%-16s %12s %-15s %s\n", m->name, buf3, buf2, m->location);
	}
}

