/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "graph.h"
#include "misc.h"

unsigned long get_value(unsigned long oval, unsigned long nval, int time_diff)
{
	unsigned long tmp;

	if (oval > nval)
		tmp = 0xffffffff - (oval - nval) + 1;
	else
		tmp = nval - oval;
	if (time_diff != 0)
		tmp = tmp/(unsigned long)time_diff;
	else
		tmp = 0;
	return tmp;
}

void Dvice::clear()
{
	grabtime = 0;
	// we should delete the interfaces before deleting the links
	interfaces.clear();
}

Graph::Graph()
{
}

Graph::~Graph()
{
}

// we load the current data and the old data files
// store in two device objects
// if an entry is missing from either leave data value in output as 0
// if status of either is down don't record data
// if both entries the output value is the difference (fix rounding errors)
// move current data overwriting old data

void Graph::main()
{
	FILE *f;
	struct passwd *p;
	char mapdir[256];
	char buf[256], host[256], ip[256], comm[256];
	int a, b;

	d1 = new Dvice();
	d2 = new Dvice();

	p = getpwuid(getuid());
	sprintf(mapdir, "%s/%s", p->pw_dir, HOME_DIR);

	sprintf(buf, "%s/devices", mapdir);
	f = fopen(buf, "r");
	if (!f) return;

	while(fscanf(f, "%[^\n]\n", buf)==1)
	{
		sscanf(buf, "%[^,],%[^,],%[^\n]", host, ip, comm);
		strcpy(name, host);
		sprintf(buf, "%s/raw/%s", mapdir, name); a = load(d1, buf);
		sprintf(buf, "%s/raw/%s.old", mapdir, name); b = load(d2, buf);
		if ((a==1) && (b==1))
		{
//			print();
			save();
		}
		sprintf(buf, "cp %s/raw/%s %s/raw/%s.old 2>/dev/null", mapdir, name, mapdir, name);
		system(buf);
	}

	fclose(f);
}

int Graph::load(Dvice *dv, char *fname)
{
	FILE *f;
	char buf[256], var[256], val[256];

//	printf("Loading %s\n", fname);

	f = fopen(fname, "r");
	if (!f) return 0;

	dv->clear();

	fscanf(f, "%[^\n]\n", buf);
	dv->grabtime = make_time_from_string(buf);
	while(fscanf(f, "%[^\n]\n", buf)==1)
	{
		if (sscanf(buf, "%[^ ] %[^\n]", var, val)==2)
		{
			if (starts_with(".iso.3.6.1.2.1.2.2.1", var))
			{
				add_to_interface(var, val, dv);
			}
		}
	}

	if (dv->grabtime==0) return 0;

	fclose(f);
	return 1;
}

void Graph::add_to_interface(char *var, char *val, Dvice *dv)
{
	char vartype[256];
	unsigned long s;
	int ifnum;
	Iface *iface, *iffound;
	Link *ln;

	// start by spliting the variable up
	if (sscanf(var, ".iso.3.6.1.2.1.2.2.1.%[^.].%d", vartype, &ifnum)!=2)
		return;

	// find an interface with same ifnum
	iffound = NULL;
	for (ln=dv->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		iface = (Iface*) ln->item;
		if (iface->index == ifnum) iffound = iface;
	}
	if (iffound == NULL)
	{
		iface = new Iface();
		iface->index = ifnum;
		dv->interfaces.tag(iface);
	}else
		iface = iffound;

	if (strcmp("10", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->octin = s;
	}else
	if (strcmp("12", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->bcastin = s;
	}else
	if (strcmp("13", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->disin = s;
	}else
	if (strcmp("14", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->errin = s;
	}else
	if (strcmp("16", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->octout = s;
	}else
	if (strcmp("18", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->bcastout = s;
	}else
	if (strcmp("19", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->disout = s;
	}else
	if (strcmp("20", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			iface->errout = s;
	}
}

void Graph::print()
{

/*

	int i;
	Iface *ifc;

	printf("\n\nDevice %s\n", name);
	printf("New time %s, %d interfaces\n", d1->grabtime, d1->interfaces.size());
	for (i=0; i<d1->interfaces.size(); i++)
	{
		ifc = (Iface*) d1->interfaces.get_item(i);
		printf("Index %8d, octin %12u, octout %12u, diff octin %12u\n", ifc->index, ifc->octin, ifc->octout, ifc->octin - ((Iface*)d2->interfaces.get_item(i))->octin);
	}
	printf("Old time %s, %d interfaces\n", d2->grabtime, d2->interfaces.size());
	for (i=0; i<d2->interfaces.size(); i++)
	{
		ifc = (Iface*) d2->interfaces.get_item(i);
		printf("Index %8d, octin %12u, octout %12u\n", ifc->index, ifc->octin, ifc->octout);
	}

*/

}

void Graph::save()
{
	int i;
	char val[256];
	struct stat statbuf;
	struct tm *pnew_time, *pold_time;
	time_t new_time;		// time stamp of current data
	time_t old_time;		// time stamp of old data
	Iface *nif, *oif;
	FILE *f;
	unsigned long tmp, size, offset;
	int time_diff;
	struct passwd *p;
	char mapdir[256];
	char buf[8];
	Link *ln, *ln2;

	p = getpwuid(getuid());
	sprintf(mapdir, "%s/%s", p->pw_dir, HOME_DIR);

	new_time = d1->grabtime;
	old_time = d2->grabtime;
	pold_time  = gmtime(&old_time);
	pnew_time  = gmtime(&new_time);

	ln  = d2->interfaces.get_head();
	ln2 = d1->interfaces.get_head();

	while ((ln!=NULL) && (ln2!=NULL))
	{
		oif = (Iface*) ln->item;
		nif = (Iface*) ln2->item;

		if ((oif!=NULL) && (nif!=NULL))
		{
			// check directories exist
			sprintf(val, "%s/graph", mapdir);
			if (stat(val, &statbuf) == -1) mkdir(val, 493); // rwxr xr x
			sprintf(val, "%s/graph/%s", mapdir, name);
			if (stat(val, &statbuf) == -1) mkdir(val, 493); // rwxr xr x

			sprintf(val, "%s/graph/%s/%d", mapdir, name, nif->index);
			if ((stat(val, &statbuf) == -1) || (statbuf.st_size == 0))
			{
				// no such file so create a new empty file
				f = fopen(val, "w");
				if (f)
				{
					size = 40*12*24*3;
					strcpy(buf, "V1.0"); for (i=0; i<4; i++) fputc(buf[i], f);
					write_ulong(f, size);					// how much data before wrap
					write_ulong(f, 12);						// offset to start of data
					for (i=0; i<(long)size; i++) fputc(0, f);
					fclose(f);
				}
			}

			// read the offset for new data
			f = fopen(val, "r+");
			if (f)
			{
				fseek(f, 8L, SEEK_SET);
				offset = read_ulong(f);
				fseek(f, offset, SEEK_SET);

				write_time_t(f, new_time);
				write_ulong(f, 0);

				time_diff = new_time - old_time;
				tmp = get_value(oif->octin, nif->octin, time_diff);
				write_ulong(f, tmp);
				tmp = get_value(oif->octout, nif->octout, time_diff);
				write_ulong(f, tmp);
				tmp = get_value(oif->bcastin, nif->bcastin, time_diff);
				write_ulong(f, tmp);
				tmp = get_value(oif->bcastout, nif->bcastout, time_diff);
				write_ulong(f, tmp);
				tmp = get_value(oif->disin, nif->disin, time_diff);
				write_ulong(f, tmp);
				tmp = get_value(oif->disout, nif->disout, time_diff);
				write_ulong(f, tmp);
				tmp = get_value(oif->errin, nif->errin, time_diff);
				write_ulong(f, tmp);
				tmp = get_value(oif->errout, nif->errout, time_diff);
				write_ulong(f, tmp);

				fseek(f, 8L, SEEK_SET);
				offset = offset+40;
				if (offset+12 == size)
					offset = 12;
				write_ulong(f, offset);

				fclose(f);
			}
		}
		ln = ln->next;
		ln2 = ln2->next;
	}
}

