/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <pwd.h>
#include "process.h"
#include "misc.h"
#include "topology.h"
#include "graph.h"
#include "ping.h"

#define BUFSIZE 256

Process::Process()
{
	tid = 0;
}

void Process::child()
{
	struct passwd *p;
	char mapdir[256];
	int i, quit = 0;
	char buf[BUFSIZE];
	char msg[BUFSIZE];
	char str[256];
	Topology *tp;
	Graph *gp;
	Ping *pg;

	p = getpwuid(getuid());
	sprintf(mapdir, "%s/%s", p->pw_dir, HOME_DIR);

	fcntl(fd[0], F_SETFL, O_NONBLOCK);

	while(!quit)
	{
		i = read(fd[0], buf, BUFSIZE);
		if (i>0)
		{
			sprintf(msg, "DONE %s", buf);
			if (starts_with("nomad_statistics", buf))
			{
				sprintf(str, "cd %s ; nice -n 19 %s", mapdir, buf);
				system(str);
			}else
			if (starts_with("nomad_topology", buf))
			{
				sprintf(str, "cd %s ; nice -n 19 %s", mapdir, buf);
				system(str);
			}else
			if (strcmp(buf, "gen graph")==0)
			{
				gp = new Graph();
				gp->main();
				delete gp;
			}else
			if (strcmp(buf, "gen network")==0)
			{
				tp = new Topology();
				tp->main();
				delete tp;
			}else
			if (strcmp(buf, "nomad_alive")==0)
			{
				sprintf(str, "cd %s ; nice -n 19 %s", mapdir, buf);
				system(str);
				pg = new Ping();
				pg->main();
				delete pg;
			}else
			if (strcmp(buf, "quit")==0) quit = 1; else
				sprintf(msg, "FAILED %s", buf);
			write(fd[0], msg, strlen(msg)+1);
		}
		usleep(2000000);
	}
}

int Process::create()
{
	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fd)<0)
		return 0;

	pid = fork();
	if (pid == -1)
	{
		return 0;
	}else
	if (pid == 0)
	{
		close(fd[1]);
		child();
		close(fd[0]);
		_exit(0);
	}

	fcntl(fd[1], F_SETFL, O_NONBLOCK);

	return 1;
}

void Process::send(char *str)
{
	write(fd[1], str, strlen(str)+1);
}

int Process::get(char *str)
{
	int i;
	char buf[BUFSIZE];

	i = read(fd[1], buf, BUFSIZE);
	if (i>0)
	{
		strcpy(str, buf);
		return 1;
	}
	return 0;
}

int Process::check_link()
{
	return (kill(pid, 0));
}

