/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#ifndef TOPOLOGY_H
#define TOPOLOGY_H 1

#include "data.h"
#include "list.h"
#include "ipaddress.h"

class Network;
class Device;
class Port;
class Machine;
class Iface;
class Interface;
class Subnet;
class Router;
class Topology
{
	public:
		Topology();
		~Topology();

		void main();
		void load(char *fname);
		void add_to_interface(char *var, char *val);
		void add_to_interface_extended(char *var, char *val);
		void add_to_route(char *var, char *val);
		void add_to_ips(char *var, char *val);
		void add_to_address(char *var, char *val);
		void add_bridge_map(char *var, char *val);
		void add_to_bridge(char *var, char *val);
		void port_to_if();

		void add_subnet(Subnet *s);
		void build_network();
		void join_networks(Interface *in1, Interface *in2);
		void build_tricky(Interface *in, List *s);
		Machine *lookup_mac_machine(unsigned char *mac);
		Interface *lookup_mac_router(unsigned char *mac);
		int mac_match_all(Machine *m, Network *n);
		void move_machine(Machine *m1, Interface *in2);
		void add_machine(Interface *in3, unsigned char *mac, IPAddress ip);
		void build_machines();
		Subnet *find_subnet(IPAddress ip);
		void merge_networks(Interface *in1, Interface *in2);
		Network *lookup_machine(Machine *m);
		void merge();
		void merge_machines_file();
		void merge_subnets_file();
		void save();

		Data newdata;
		Data olddata;
		Port *pt;
		int num_ports;
		int mm;
		char mapdir[256];

		int pos;		// current device
		Device *dv;
		int num_devices;

	private:
		Device *get_device(Router *r);
		Iface *get_interface(Interface *in);
};

#endif

